function result = EpDel(cmd)
% Epsilon-Delta Continuity
%
% A module to illustrate the  epsilon-delta definition of continuity.
%
% Choose the domain of x values, a function f(x), and a point x0 in
% in the domain.  Then, pick an epsilon.  Then computer will show the
% neighborhood f(x0) - epsilon  y  f(x0) + epsilon.  Now, you can
% click and drag in the graph window to find a delta small enough so
% that x0 - delta  x  x0 + delta implies that f(x0) is in the 
% neighborhood around f(x0).
%
% 2.1/22-July-1998

% HISTORY:
% 1.0 -- Original by Julie Mitchell (29-Nov-1996).
% 2.0 -- Rewritten by Tom Insel, Spring 1998
% 2.1 -- For MATLAB 5, can change delta, etc.
%
% USES: cleanup.m, errmesg.m
%
% very much like tomtanline

persistent fig_axes epdel_line...
	f_of_x xlow_field xhi_field x0_field delta_input epsilon_input

math_bkgnd = [0.6 0.8 1.0];
math_field = [1 1 1];
math_epcolor = [.5 .5 .5];
math_delcolor = [.7 0 0];

if ~nargin,

	control_figures = findobj('tag', 'epdel_main');
	
	if length(control_figures) == 0
		cmd = 'new';
	else
		figure(control_figures(1));
		cmd = 'NOCOMMAND';
	end

end

if strcmp(cmd, 'new')

	control_figures = findobj('tag', 'epdel_main');
	
	if length(control_figures) == 0	
		epdel('makemain')
	else
		figure(control_figures(1))
	end
	
	epdel('draw')
	
elseif strcmp(cmd,'makemain')

	fig_han = figure(...
		'name', 'Epsilon-Delta Continuity',...
		'Tag', 'epdel_main',...
		'numbertitle', 'off',...
		'position', [10 10 600 400],...
		'resize', 'off',...
		'backingstore', 'off',...
		'color', math_bkgnd,...
		'WindowButtonUpFcn',...
			'set(gcf,''WindowButtonMotionFcn'','''',''Pointer'',''arrow'');',...
		'WindowButtonDownFcn',...
			'set(gcf,''WindowButtonMotionFcn'',''EpDel(99)'',''Pointer'',''crosshair'');EpDel(99);');

	uicontrol(fig_han,...
		'style', 'frame',...
		'backgroundcolor', math_bkgnd,...
		'position', [10 105 580 1]);

	uicontrol(fig_han,...
		'style', 'text',...
		'horizontalalignment', 'right',...
		'string', 'f(x) =',...
		'backgroundcolor', math_bkgnd,...
		'position', [10 70 45 20]);

	f_of_x = uicontrol(fig_han,...
		'style', 'edit',...
		'backgroundcolor', math_field,...
		'string', 'x^3 - 2*x',...
		'position', [60 70 200 20],...
		'callback', 'EpDel(''draw'')');

	xlow_field = uicontrol(fig_han,...
		'style', 'edit',...
		'backgroundcolor', math_field,...
		'string', '-2',...
		'position', [60 45 50 20],...
		'callback', 'EpDel(''draw'')');
		
	if strcmp(computer,'MAC2')
		domain_string = ' x ';
	else
		domain_string = '<= x <=';
	end
		
	uicontrol(fig_han,...
		'style', 'text',...
		'string', domain_string,...
		'backgroundcolor', math_bkgnd,...
		'position', [110 45 40 20]);

	xhi_field = uicontrol(fig_han,...
		'style', 'edit',...
		'backgroundcolor', math_field,...
		'string', '2',...
		'position', [150 45 50 20],...
		'callback', 'EpDel(''draw'')');
		
	uicontrol(fig_han,...
		'style', 'text',...
		'horizontalalignment', 'right',...
		'string', 'x0 =',...
		'backgroundcolor', math_bkgnd,...
		'position', [10 20 45 20]);
		
	x0_field = uicontrol(fig_han,...
		'style', 'edit',...
		'backgroundcolor', math_field,...
		'string', '0',...
		'position', [60 20 50 20],...
		'callback', 'EpDel(''draw'')');

	uicontrol(fig_han,...
		'style', 'text',...
		'horizontalalignment', 'right',...
		'string', 'epsilon =',...
		'backgroundcolor', math_bkgnd,...
		'position', [300 70 50 20]);
		
	epsilon_input = uicontrol(fig_han,...
		'style', 'edit',...
		'backgroundcolor', math_field,...
		'string', '.5',...
		'position', [360 70 100 20],...
		'callback', 'EpDel(''draw'')');
		
	uicontrol(fig_han,...
		'style', 'text',...
		'horizontalalignment', 'right',...
		'string', 'delta =',...
		'backgroundcolor', math_bkgnd,...
		'position', [300 45 50 20]);
		
	delta_input = uicontrol(fig_han,...
		'style', 'edit',...
		'backgroundcolor', math_field,...
		'string', '1',...
		'callback', 'EpDel(''draw'')',...
		'position', [360 45 100 20]);

	uicontrol(fig_han,...
		'string', 'Help',...
		'position', [510 55 80 35],...
		'callback', 'helpwin EpDel');

	uicontrol(fig_han,...
		'string', 'Quit',...
		'position', [510 10 80 35],...
		'callback', 'EpDel(''end'');');
		
	fig_axes = axes(...
		'units', 'normalized',...
		'position', [.05 .33 .9 .6],...
		'XLimMode', 'auto',...
		'YLimMode', 'auto');
	
	if nargout, result = fig_han, end
			
elseif strcmp(cmd,'draw') | cmd == 99

	main_figures = findobj('tag', 'epdel_main');
	if length(main_figures) == 0
		main_figures = EpDel('makemain');
	end

	main_figure = main_figures(1);
			
	try
		xlow = str2num(cleanup(get(xlow_field,'string')));
		xhi = str2num(cleanup(get(xhi_field,'string')));
	catch
		errmesg('x domain', 'num')
		return
	end

	if xlow > xhi
		temp = xlow;
		xlow = xhi;
		xhi = temp;
	end
		
	try
		x0 = str2num(cleanup(get(x0_field,'string')));	
	catch
		errmesg('x0', 'num')
		return
	end
	
	if x0 > xhi
		x0 = xhi;
		set(x0_field,'string',num2str(x0))
	elseif x0 < xlow
		x0 = xlow;
		set(x0_field,'string',num2str(x0))
	end

	try
		epsilon = str2num(cleanup(get(epsilon_input, 'string')));
	catch
		errmesg('epsilon', 'num')
		return
	end		
	
	if cmd == 99
		
		p = get(fig_axes, 'CurrentPoint');
		x1  = p(1, 1);
		x = x1;
		
		try
			y1 = eval(vectorize(cleanup(get(f_of_x, 'string'))));
		catch
			errmesg('f(x)', 'fun'), return
		end
		
		delta = abs(x1 - x0);
		set(delta_input, 'string', num2str(delta))
		
		ylims = get(fig_axes, 'ylim');	
		set(epdel_line(1), 'xdata', [x0-delta x0-delta], 'ydata', ylims);
		set(epdel_line(2), 'xdata', [x0+delta x0+delta], 'ydata', ylims);
			
	else

		try
			x = x0;
			y0 = eval(vectorize(cleanup(get(f_of_x, 'string'))));
			x = linspace(xlow,xhi,600);
			y = eval(vectorize(cleanup(get(f_of_x, 'string'))));
		catch
			errmesg('f(x)', 'fun'), return
		end
	
		plot(x, y);
		
		try
			delta = str2num(get(delta_input, 'string'));
		catch
			errmesg('delta', 'num'), return
		end

		hold on
		plot([xlow xhi], [y0+epsilon y0+epsilon], 'color', math_epcolor)
		plot([xlow xhi], [y0-epsilon y0-epsilon], 'color', math_epcolor)
		ylims = get(fig_axes, 'ylim');	
		epdel_line = [...
			plot([x0-delta x0-delta],ylims,'color',math_delcolor,'erasemode','xor')...
			plot([x0+delta x0+delta],ylims,'color',math_delcolor,'erasemode','xor')];
		plot(x0,y0,'r*')
		hold off
		
		title('Click & Drag to Select delta')
	end

elseif strcmp(cmd, 'end')

	w = findobj('tag', 'epdel_main');
	for i = 1:length(w)
		delete(w(i))
	end

	% if it's open, return to the main menu

	w = findobj('tag', 'math_main_menu');
	if length(w)
		figure(w(1));
	end

end;
