% ADJOINT
%
% This module computes the adjoint of a square matrix A.
% To use it select the size of A, then type in A and click
% the compute button.  The adjoint of A and the product
% A * Adj(A) are then displayed.  To see how the adjoint 
% changes when a single entry of A is changed, select the 
% Auto checkbox.  

% AUTHOR: William A. Stein
% May, 1998

function Adjoint (action)
persistent h_fig h_r h_a h_auto h_adj h_aadj ...
   size auto 
if nargin == 0
   action = 'init' ;
end
switch action
case 'init',
   h_fig = findobj ('tag','Adjoint') ;
   if isempty (h_fig)
      a = AdjointWin ;
      h_fig = a{1} ;
      h_r = a{2} ;
      h_auto = a{3} ; 
      h_a = a{4} ;
      h_adj = a{5} ; 
      h_aadj = a{6} ;
      auto = 0 ;
      size = 2 ;
      Adjoint 'refresh' 
   else
      figure (h_fig (1)) 
   end
   
case 'examples',
   e=[];
   e(1).button = 'Question 1e' ;
   e(1).text = 'A 3x3 matrix.' ;
   e(1).ex = [1 2 5; 2 3 -1; 1 0 1] ; 
   e(1).size = 3 ;
   i = examples('Gauss', e) ;
   if i 
	   size = e(i).size ;
      for r=1:size
         for c=1:size
            set(h_a(r,c),'string',num2str(e(1).ex(r,c)))
         end
      end
      auto = 1 ;
      Adjoint refresh
   end
	
   
case 'matrix',
   if auto
      Adjoint 'compute'
   end
   
case 'radio2',
   size=2;    
   auto=0;
   Adjoint 'refresh';
case 'radio3',
   size=3;
   auto=0;
   Adjoint 'refresh';
case 'radio4',
   size=4;
   auto=0;
   Adjoint 'refresh';
case 'radio5',
   size=5;
   auto=0;
   Adjoint 'refresh';
   
case 'auto',
   auto=get(h_auto,'value');
   
case 'help',
   helpwin('Adjoint')
   
case 'quit',
   delete (findobj ('tag', 'Adjoint')) ;
   w=findobj ('tag', 'math_main_menu') ;
   if length(w)
      figure(w(1));
   end
   
case 'refresh',
   for i=2:5   % radio buttons
      set (h_r(i), 'value', size==i)
   end
   set (h_auto, 'value', auto)   % auto refresh button
   % matrices
   for i=2:size
      set (findobj (h_fig, 'tag', num2str(i)), 'visible', 'on') 
   end
   for i=size+1:5
      set (findobj (h_fig, 'tag', num2str(i)), 'visible', 'off') 
   end
   if auto
      Adjoint 'compute'
   else
      set ([h_adj h_aadj], 'string', '.') 
      set (h_a, 'string', '0') ;
   end
   
   
case 'compute',
   try
      for i=1:size
         for j=1:size
            %            a(i,j)=sym(cleanup(get(h_a(i,j),'string')));
            a(i,j)=str2num(cleanup(get(h_a(i,j),'string')));
         end
      end
      for i=1:size
         for j=1:size
            adj(j,i)=(-1)^(i+j)*det(a([1:i-1 i+1:size],[1:j-1 j+1:size]));
         end
      end
      aadj=a*adj;
      for i=1:size
         for j=1:size
%           set(h_adj(i,j),'string',char(adj(i,j)));
%				set(h_aadj(i,j),'string',char(aadj(i,j)));
            set(h_adj(i,j),'string',num2str(adj(i,j)));
				set(h_aadj(i,j),'string',num2str(aadj(i,j)));
         end
      end
   catch
      errordlg(['An entry, probably (' num2str(i) ',' num2str(j) '), is not valid.  Please correct.'], 'Input Error');
   end
end

function ret = AdjointWin()

h0 = figure('Color',[0.6 0.8 1], ...
	'Name','ADJOINT', ...
	'NumberTitle','off', ...
	'Position',[178   50   600   400], ...
	'Resize','off', ...
   'Tag','Adjoint', ...
   'Menubar', 'none');

r2 = uicontrol('Parent',h0, ...
	'BackgroundColor',[0.6 0.8 1], ...
	'Callback','Adjoint (''radio2'')', ...
	'Position',[25 360 49.5 15], ...
	'String','2 x 2', ...
	'Style','radiobutton');

r3 = uicontrol('Parent',h0, ...
	'BackgroundColor',[0.6 0.8 1], ...
	'Callback','Adjoint (''radio3'')', ...
	'Position',[25 330 50 15], ...
	'String','3 x 3', ...
	'Style','radiobutton');

r4 = uicontrol('Parent',h0, ...
	'BackgroundColor',[0.6 0.8 1], ...
	'Callback','Adjoint(''radio4'')', ...
	'Position',[25 300 50 15], ...
	'String','4 x 4', ...
	'Style','radiobutton');

r5 = uicontrol('Parent',h0, ...
	'BackgroundColor',[0.6 0.8 1], ...
	'Callback','Adjoint(''radio5'')', ...
	'Position',[25 270 50 15], ...
	'String','5 x 5', ...
	'Style','radiobutton');

auto = uicontrol('Parent',h0, ...
	'BackgroundColor',[0.6 0.8 1], ...
   'Callback','Adjoint (''auto'')', ...
   'HorizontalAlignment','left',...
	'Position',[25 240 100 15], ...
	'String','Auto Refresh', ...
	'Style','checkbox');


for i = 1:5
   for j = 1:5
      x = 165 + (j-1)*36 ;
      y = - (i-1)*20 ;
      pos = num2str (5*(i-1) + j) ;
      tag = num2str (max(i,j)) ;
      
      Ma(i,j) = uicontrol('Parent',h0, ...
         'BackgroundColor',[1 1 1], ...
         'Callback','Adjoint (''matrix'')', ...
         'Position',[x+110 y+310 37 20], ...
         'String', '', ...
         'Style','edit', ...
         'Visible', 'off', ...
         'Tag', tag);
      
      Madj(i,j) = uicontrol('Parent',h0, ...
         'BackgroundColor',[1 1 1], ...
         'Position',[x y+120 37 20], ...
         'String',pos, ...
         'Style','text', ...
         'Visible', 'off', ...
         'Tag', tag);
      
      Maadj(i,j) = uicontrol('Parent',h0, ...
         'BackgroundColor',[1 1 1], ...
         'Position',[x+220 y+120 37 20], ...
         'String',pos, ...
         'Style','text', ...
         'Visible', 'off', ...
         'Tag', tag);
  end
end

set ([Ma(1,1) Madj(1,1) Maadj(1,1)], 'visible', 'on') ;

uicontrol('Parent',h0, ...
	'BackgroundColor',[0.6 0.8 1], ...
	'Position',[230 310 27 20.25], ...
	'String','A = ', ...
	'Style','text');

uicontrol('Parent',h0, ...
	'BackgroundColor',[0.6 0.8 1], ...
	'Position',[165 145 55 20], ...
	'String','adj(A) = ', ...
	'Style','text');

uicontrol('Parent',h0, ...
	'BackgroundColor',[0.6 0.8 1], ...
	'Position',[385 145 73 20], ...
	'String','A adj(A) = ', ...
	'Style','text');

uicontrol('Parent',h0, ...
	'Callback','Adjoint(''compute'')', ...
	'Position',[20 140 110 30], ...
	'String','Compute');

uicontrol('Parent',h0, ...
	'Callback','Adjoint (''help'')', ...
	'Position',[20 100 110 30], ...
   'String','Help');

uicontrol('Parent',h0, ...
	'Callback','Adjoint(''examples'')', ...
	'Position',[20 60 110 30], ...
	'String','Examples');

uicontrol('Parent',h0, ...
	'Callback','Adjoint (''quit'')', ...
   'Position',[20 20 110 30], ...
	'String','Quit');

if nargout > 0, 
   ret = { h0, [0, r2, r3, r4, r5], ...
         auto, Ma, Madj, Maadj }; 
end
