function result = cardboard(cmd, arg)
% CARDBOARD
%
% This module is for investigating the behavior of a function f(x,y,z)
% subject to the constraint that x*y*z remain constant.  A typical
% application is to watch the surface area of a box with fixed volume.
%
% Enter starting x, y, and z values into the appropriate boxes. 
% Enter the function f(x,y,z) that you want to monitor.  For example,
% surface area would be 2*(x*y + y*z + z*x).
%
% Now, use the STRETCH and SHRINK buttons to increase or decrease
% one variable, while changing the other variables to keep x*y*z 
% constant.
%
% 1.0/18-July-1998

% HISTORY:
% 1.0 -- original by Tom Insel, June 1998
%
% USES: cleanup.m, errmesg.m
%
% Tested: MATLAB 5.2 for Macintosh

persistent fig_axes x_field y_field z_field volume_text f_field f_text

math_bkgnd = [0.6 0.8 1.0];
math_field = [1 1 1];
math_epcolor = [.7 0 0];

ratio = 1.2 ;

if ~nargin,

	control_figures = findobj('tag', 'cardboard_main');
	
	if length(control_figures) == 0
		cmd = 'new';
	else
		figure(control_figures(1));
		cmd = 'NOCOMMAND';
	end

end

if strcmp(cmd,'new')

	fig_han = figure(...
		'name', 'Cardboard Box',...
		'Tag', 'cardboard_main',...
		'numbertitle', 'off',...
		'position', [10 10 600 400],...
		'color', math_bkgnd,...
		'resize', 'off',...
		'backingstore', 'off');

	uicontrol(fig_han,...
		'style', 'frame',...
		'position', [200 10 1 380]);

	uicontrol(fig_han,...
		'style', 'text',...
		'horizontalalignment', 'left',...
		'string', 'BOX DIMENSIONS',...
		'backgroundcolor', math_bkgnd,...
		'position', [10 370 180 20]);

	uicontrol(fig_han,...
		'style', 'text',...
		'horizontalalignment', 'right',...
		'string', 'x =',...
		'backgroundcolor', math_bkgnd,...
		'position', [20 345 30 20]);
		
	x_field = uicontrol(fig_han,...
		'style', 'edit',...
		'horizontalalignment', 'left',...
		'string', '10',...
		'background', math_field,...
		'callback','cardboard(''draw'');',...
		'position', [60 345 80 20]);

	uicontrol(fig_han,...
		'style', 'text',...
		'horizontalalignment', 'right',...
		'string', 'y =',...
		'backgroundcolor', math_bkgnd,...
		'position', [20 320 30 20]);
		
	y_field = uicontrol(fig_han,...
		'style', 'edit',...
		'horizontalalignment', 'left',...
		'string', '10',...
		'background', math_field,...
		'callback','cardboard(''draw'');',...
		'position', [60 320 80 20]);
		
	uicontrol(fig_han,...
		'style', 'text',...
		'horizontalalignment', 'right',...
		'string', 'z =',...
		'backgroundcolor', math_bkgnd,...
		'position', [20 295 30 20]);
		
	z_field = uicontrol(fig_han,...
		'style', 'edit',...
		'horizontalalignment', 'left',...
		'string', '10',...
		'background', math_field,...
		'callback','cardboard(''draw'');',...
		'position', [60 295 80 20]);

	volume_text = uicontrol(fig_han,...
		'style', 'text',...
		'horizontalalignment', 'left',...
		'backgroundcolor', math_bkgnd,...
		'position', [20 265 170 20]);
				
	uicontrol(fig_han,...
		'style', 'frame',...
		'position', [10 255 180 1]);

	uicontrol(fig_han,...
		'style', 'text',...
		'horizontalalignment', 'left',...
		'string', 'STRETCH',...
		'backgroundcolor', math_bkgnd,...
		'position', [10 225 50 20]);
		
	uicontrol(fig_han,...
		'string', 'x',...
		'position', [70 225 20 20],...
		'callback', 'cardboard(''grow'',''x'');');
		
	uicontrol(fig_han,...
		'string', 'y',...
		'position', [100 225 20 20],...
		'callback', 'cardboard(''grow'',''y'');');

	uicontrol(fig_han,...
		'string', 'z',...
		'position', [130 225 20 20],...
		'callback', 'cardboard(''grow'',''z'');');

	uicontrol(fig_han,...
		'style', 'text',...
		'horizontalalignment', 'left',...
		'string', 'SHRINK',...
		'backgroundcolor', math_bkgnd,...
		'position', [10 200 50 20]);
		
	uicontrol(fig_han,...
		'string', 'x',...
		'position', [70 200 20 20],...
		'callback', 'cardboard(''shrink'',''x'');');
		
	uicontrol(fig_han,...
		'string', 'y',...
		'position', [100 200 20 20],...
		'callback', 'cardboard(''shrink'',''y'');');

	uicontrol(fig_han,...
		'string', 'z',...
		'position', [130 200 20 20],...
		'callback', 'cardboard(''shrink'',''z'');');
		
	uicontrol(fig_han,...
		'style', 'frame',...
		'position', [10 190 180 1]);
		
	uicontrol(fig_han,...
		'style', 'text',...
		'horizontalalignment', 'right',...
		'string', 'f(x,y,z) =',...
		'backgroundcolor', math_bkgnd,...
		'position', [10 160 52 20]);
		
	f_field = uicontrol(fig_han,...
		'style', 'edit',...
		'horizontalalignment', 'left',...
		'string', '2*(x*y + y*z + z*x)',...
		'background', math_field,...
		'callback','cardboard(''draw'');',...
		'position', [67 160 123 20]);
		
	uicontrol(fig_han,...
		'style', 'text',...
		'horizontalalignment', 'right',...
		'string', '=',...
		'backgroundcolor', math_bkgnd,...
		'position', [10 135 52 20]);
		
	f_text = uicontrol(fig_han,...
		'style', 'text',...
		'horizontalalignment', 'left',...
		'background', math_bkgnd,...
		'position', [67 135 123 20]);
		
	uicontrol(fig_han,...
		'string', 'Help',...
		'position', [10 10 85 35],...
		'callback', 'helpwin cardboard');

	uicontrol(fig_han,...
		'string', 'Quit',...
		'position', [105 10 85 35],...
		'callback', 'cardboard(''end'');');
				
	fig_axes = axes('position', [.4 .05 .55 .9]);
	
	cardboard('draw');
	
	if nargout, result = fig_han, end
	
elseif strcmp(cmd, 'draw') | strcmp(cmd, 'grow') | strcmp(cmd, 'shrink')

	main_figures = findobj('tag', 'cardboard_main');
	if length(main_figures) == 0
		main_figures = cardboard('new');
	end

	main_figure = main_figures(1);
		
	err_field=0;
	err_type=0;
	
	try
		x0 = str2num(cleanup(get(x_field,'string')));
	catch
		errmesg('x','num')
	end

	try
		y0 = str2num(cleanup(get(y_field,'string')));
	catch
		errmesg('y','num')
	end

	try
		z0 = str2num(cleanup(get(z_field,'string')));
	catch
		errmesg('z','num')
	end
	
	if strcmp(cmd, 'draw')
		
		x = x0; y = y0; z = z0;
		
	elseif strcmp(cmd, 'grow')

		% increase one dimension while keeping volume constant

		volume = x0 * y0 * z0;

		if strcmp(arg, 'x')
			x1 = ratio*x0;
			r = volume/(x1 * y0 * z0);
			y1 = sqrt(r) * y0;
			z1 = sqrt(r) * z0;
		elseif strcmp(arg, 'y')
			y1 = ratio*y0;
			r = volume/(y1 * x0 * z0);
			x1 = sqrt(r) * x0;
			z1 = sqrt(r) * z0;
		else	
			z1 = ratio*z0;
			r = volume/(z1 * x0 * y0);
			x1 = sqrt(r) * x0;
			y1 = sqrt(r) * y0;
		end
			
		x = x1; y = y1; z = z1;
	
	elseif strcmp(cmd, 'shrink')

		volume = x0 * y0 * z0;

		if strcmp(arg, 'x')
			x1 = x0/ratio;
			r = volume/(x1 * y0 * z0);
			y1 = sqrt(r) * y0;
			z1 = sqrt(r) * z0;
		elseif strcmp(arg, 'y')
			y1 = y0/ratio;
			r = volume/(y1 * x0 * z0);
			x1 = sqrt(r) * x0;
			z1 = sqrt(r) * z0;
		else	
			z1 = z0/ratio;
			r = volume/(z1 * x0 * y0);
			x1 = sqrt(r) * x0;
			y1 = sqrt(r) * y0;
		end
			
		x = x1; y = y1; z = z1;
		
	end

	% these define a cube, as in _Using_MATLAB_Graphics_.
	
	vertices = [0 0 0; x 0 0; x y 0; 0 y 0; 0 0 z; x 0 z; x y z; 0 y z];
	faces = [1 2 6 5; 2 3 7 6; 3 4 8 7; 4 1 5 8; 1 2 3 4; 5 6 7 8];

	axes(fig_axes)
	cla

	patch('Vertices', vertices,...
		'Faces', faces,...
		'FaceVertexCData', jet(6),...
		'Facecolor', 'flat');
	view(3)
	axis vis3d;
	set(gca, 'DataAspectRatio', [1 1 1]);
	axis equal;
	axis off;
%	rotate3d ON
	xlabel('x')
	ylabel('y')
	zlabel('z')

	set(volume_text, 'string', ['Volume = x*y*z = ' num2str(x*y*z)])
	set(x_field, 'string', num2str(x))
	set(y_field, 'string', num2str(y))
	set(z_field, 'string', num2str(z))
	
	try
		set(f_text, 'string', num2str(eval(cleanup(get(f_field,'string')))))
	catch
		errmesg('f(x,y,z)', 'fun')
	end

elseif strcmp(cmd, 'end')

	w = findobj('tag', 'cardboard_main');
	for i = 1:length(w)
		delete(w(i))
	end

	% if it's open, return to the main menu

	w = findobj('tag', 'math_main_menu');
	if length(w)
		figure(w(1));
	end

end;
