function result = catenary(cmd)
% POWER LINES
%
% This module lets you visualize a catenary curve, a*cosh(x/a) + c.
% A catenary is the shape that a wire or chain makes when hanging
% with both ends attached.

% HISTORY:
% 1.0 -- Original version by Julie Mitchell (1996)
% 2.0 -- Rewritten from scratch by by Tom Insel, Spring 1998
%
% USES: cleanup.m, errmesg.m

math_bkgnd = [0.6 0.8 1.0];
math_field = [1 1 1];
math_epcolor = [.7 0 0];

if ~nargin,

	control_figures = findobj('tag', 'catenary_main');
	
	if length(control_figures) == 0
		cmd = 'new';
	else
		figure(control_figures(1));
		cmd = 'NOCOMMAND';
	end

end

if strcmp(cmd,'new')

	fig_han = figure(...
		'name', 'Power Lines',...
		'Tag', 'catenary_main',...
		'numbertitle', 'off',...
		'position', [10 10 600 400],...
		'resize', 'off',...
		'color', math_bkgnd,...
		'backingstore', 'off',...
		'WindowButtonUpFcn',...
			'set(gcf,''WindowButtonMotionFcn'','''',''Pointer'',''arrow'');',...
		'WindowButtonDownFcn',...
			'set(gcf,''WindowButtonMotionFcn'',''catenary(99)'',''Pointer'',''crosshair'');catenary(99);');

	uicontrol(fig_han,...
		'style', 'frame',...
		'position', [10 105 580 1]);

	uicontrol(fig_han,...
		'style', 'text',...
		'horizontalalignment', 'left',...
		'string', 'GRAPH',...
		'backgroundcolor', math_bkgnd,...
		'position', [10 70 250 20]);

	uicontrol(fig_han,...
		'style', 'text',...
		'horizontalalignment', 'left',...
		'string', 'f(x) = a * cosh(x/a) + c',...
		'backgroundcolor', math_bkgnd,...
		'position', [20 45 250 20]);

	uicontrol(fig_han,...
		'style', 'text',...
		'horizontalalignment', 'right',...
		'string', 'a =',...
		'backgroundcolor', math_bkgnd,...
		'position', [20 20 30 20]);
		
	a_field = uicontrol(fig_han,...
		'style', 'edit',...
		'horizontalalignment', 'left',...
		'string', '20',...
		'background', math_field,...
		'callback','catenary(''draw'');',...
		'position', [60 20 50 20]);

	uicontrol(fig_han,...
		'style', 'text',...
		'horizontalalignment', 'right',...
		'string', 'c =',...
		'backgroundcolor', math_bkgnd,...
		'position', [120 20 30 20]);
		
	c_field = uicontrol(fig_han,...
		'style', 'edit',...
		'horizontalalignment', 'left',...
		'string', '20',...
		'background', math_field,...
		'callback','catenary(''draw'');',...
		'position', [160 20 50 20]);

	uicontrol(fig_han,...
		'style', 'text',...
		'horizontalalignment', 'right',...
		'string', 'distance =',...
		'backgroundcolor', math_bkgnd,...
		'position', [220 20 60 20]);
		
	d_field = uicontrol(fig_han,...
		'style', 'edit',...
		'horizontalalignment', 'left',...
		'string', '50',...
		'callback','catenary(''draw'');',...
		'background', math_field,...
		'position', [290 20 50 20]);

	uicontrol(fig_han,...
		'string', 'Help',...
		'position', [500 55 90 35],...
		'callback', 'catenary(''help'');');

	uicontrol(fig_han,...
		'string', 'Quit',...
		'position', [500 10 90 35],...
		'callback', 'catenary(''end'');');
		
	fig_axes = axes(...
		'units', 'normalized',...
		'position', [.05 .33 .9 .6],...
		'XLimMode', 'auto',...
		'YLimMode', 'auto');

	d = [fig_axes; a_field; c_field; d_field];
	set(fig_han, 'userdata', d);
	
	catenary('draw');
	
	if nargout, result = fig_han, end
			
elseif strcmp(cmd,'draw') | cmd == 99

	main_figures = findobj('tag', 'catenary_main');
	if length(main_figures) == 0
		main_figures = catenary('new');
	end

	main_figure = main_figures(1);
	
	d = get(main_figure, 'userdata');
	fig_axes = d(1);
	a_field = d(2);
	c_field = d(3);
	d_field = d(4);
	
	err_field=0;
	err_type=0;
	
	a_string = cleanup(get(a_field,'string'));
	c_string = cleanup(get(c_field,'string'));
	d_string = cleanup(get(d_field,'string'));
	
	if cmd ~= 99
		eval([a_string ';'], 'err_field=''a''; err_type=''num'';')
		eval([c_string ';'], 'err_field=''c''; err_type=''num'';')
		eval([d_string ';'], 'err_field=''distance''; err_type=''num'';')
		if err_field
			errmesg(err_field, err_type);
			error(lasterr);
			return
		end
	end

	a = str2num(a_string);
	c = str2num(c_string);
	d = str2num(d_string);

	xhi = d/2;
	xlow = -xhi;
	
	f_string = 'a*cosh(x/a)+c';
		
	if cmd == 99
		% adjust one or more of the parameters dynamically
	else
		x = linspace(xlow,xhi,100);
		y = eval(f_string);
		plot(x, y);
		
		x = xlow;
		y0 = eval(f_string);
		
		x = xhi;
		y1 = eval(f_string);

		hold on
		plot([xlow xlow], [0 y0], 'color', math_epcolor)
		plot([xhi xhi], [0 y1], 'color', math_epcolor)
		xd = abs(xlow-xhi);
		axis([xlow-0.1*xd xhi+0.1*xd 0 1.2*max([y0 y1])]);
		hold off
	end

elseif strcmp(cmd,'help')

	helpwin catenary

elseif strcmp(cmd, 'end')

	w = findobj('tag', 'catenary_main');
	for i = 1:length(w)
		delete(w(i))
	end

	% if it's open, return to the main menu

	w = findobj('tag', 'math_main_menu');
	if length(w)
		figure(w(1));
	end

end;
