function eigenvis(cmd)
% EIGENVISUALIZER 
%
% Use this module examine the geometry of 2x2 linear transformations
%
% Enter a 2x2 transformation matrix A in the boxes at the left of the
% screen. The determinant of the matrix will be displayed immediately
% below. Click and drag in the window to choose a unit vector v.  You
% will see the vector v in red and the vector A*v in blue.
%
% Note that the vector v is an eigenvector if and only if it is
% parallel to the image vector A*v. The sign of the eigenvector
% is determined by whether these vectors overlap, and its magnitude
% is the ratio of the norms of A*v and v.
%
% 2.1/18-July-1998

% HISTORY:
% 1.0 -- Original by Julie Mitchell (29-Nov-1996).
% 2.0 -- Revised by Tom Insel, Spring 1998, with new style interface,
%        error detection, and dynamic update of vectors inspired by 
%        Project ATLAST.
% 2.1 -- MATLAB 5 and style, handles eigenvalues < 1 better, x has a
%        starting value and is maintained when T changes, removed
%        translation & compass options.
%
% USES: cleanup.m, errmesg.m

persistent vline Tvline eigen_axes
persistent T Tmatrix % the transformation matrix & handles to its fields
persistent det_text v_text Tv_text normTv_text
persistent save_x % for redraw when data changes

math_bkgnd = [0.6 0.8 1.0];
math_field = [1 1 1];

if ~nargin,

	control_figures = findobj('tag', 'eigenv_main');
	
	if length(control_figures) == 0
		cmd = 'new';
	else
		figure(control_figures(1));
		cmd = 'NOCOMMAND';
	end

end

if cmd == 99 % Update the vectors.

	p = get(eigen_axes,'CurrentPoint');
	x  = p(1, 1:2);
	x  = x / norm(x) + [0 0];
	save_x = x;
	Tx = T * x' + [0;0];
	
	set(vline, 'XData', [0 x(1)], 'YData', [0 x(2)]);
	set(Tvline, 'XData', [0 Tx(1)], 'YData', [0 Tx(2)]);
	
	set(v_text, 'string', ['v = [' num2str(x(1),4) ',' num2str(x(2),4) ']']);
	set(Tv_text, 'string', ['A*v = [' num2str(Tx(1),4) ',' num2str(Tx(2),4) ']']);
	set(normTv_text, 'string', ['norm(A*v) = ' num2str(norm(Tx)) ]);

elseif strcmp(cmd,'new')

	mymat = [2 0; 0 1]; % initial matrix

	fig_han = figure(...
		'name', 'EigenVisualizer',...
		'Tag', 'eigenv_main',...
		'numbertitle', 'off',...
		'position', [10 10 600 400],...
		'resize', 'off',...
		'backingstore', 'off',...
		'color', 'w',...
		'WindowButtonUpFcn',...
			'set(gcf,''WindowButtonMotionFcn'','''',''Pointer'',''arrow'');',...
		'WindowButtonDownFcn',...
			'set(gcf,''WindowButtonMotionFcn'',''eigenvis(99)'',''Pointer'',''crosshair'');eigenvis(99);');

	uicontrol(fig_han,...
		'style', 'frame',...
		'backgroundcolor', math_bkgnd,...
		'position', [0 0 170 400]);
		
	%
		
	uicontrol(fig_han,...
		'style', 'frame',...
		'backgroundcolor', 'k',...
		'position', [10 265 150 1]);
		
	det_text = uicontrol(fig_han,...
		'style', 'text',...
		'position', [10 360 150 20],...
		'backgroundcolor', math_bkgnd,...
		'HorizontalAlign', 'center');

	v_text = uicontrol(fig_han,...
		'style', 'text',...
		'position', [10 335 150 20],...
		'backgroundcolor', math_bkgnd,...
		'HorizontalAlign', 'center',...
		'foregroundcolor','r');

	uicontrol(fig_han,...
		'style', 'text',...
		'string', 'norm(v) = 1',...
		'position', [10 315 150 20],...
		'backgroundcolor', math_bkgnd,...
		'HorizontalAlign', 'center',...
		'foregroundcolor', 'r');
		
	Tv_text = uicontrol(fig_han,...
		'style', 'text',...
		'position', [10 290 150 20],...
		'backgroundcolor', math_bkgnd,...
		'HorizontalAlign', 'center',...
		'foregroundcolor', 'b');

	normTv_text = uicontrol(fig_han,...
		'style', 'text',...
		'position', [10 270 150 20],...
		'backgroundcolor', math_bkgnd,...
		'HorizontalAlign', 'center',...
		'foregroundcolor', 'b');
	
	%

	uicontrol(fig_han,...
		'style', 'frame',...
		'backgroundcolor', 'b',...
		'position', [10 230 150 1]);

	uicontrol(fig_han,...
		'style', 'text',...
		'string', 'TRANSFORMATION',...
		'position', [10 200 100 20],...
		'horizontalalign', 'left',...
		'backgroundcolor', math_bkgnd);

	uicontrol(fig_han,...
		'style', 'text',...
		'string', 'A = ',...
		'position', [5 165 40 20],...
		'horizontalalign', 'right',...
		'backgroundcolor', math_bkgnd);

	Tmatrix(1) = uicontrol(fig_han,...
		'style', 'edit',...
		'string', num2str(mymat(1,1)),...
		'position', [45 175 53 20],...
		'callback', 'eigenvis(''draw'');',...
		'backgroundcolor', math_field);

	Tmatrix(2) = uicontrol(fig_han,...
		'style', 'edit',...
		'string', num2str(mymat(1,2)),...
		'position', [98 175 53 20],...
		'callback', 'eigenvis(''draw'');',...
		'backgroundcolor', math_field);

	Tmatrix(3) = uicontrol(fig_han,...
		'style', 'edit',...
		'string', num2str(mymat(2,1)),...
		'position', [45 155 53 20],...
		'callback', 'eigenvis(''draw'');',...
		'backgroundcolor', math_field);

	Tmatrix(4) = uicontrol(fig_han,...
		'style', 'edit',...
		'string', num2str(mymat(2,2)),...
		'position', [98 155 53 20],...
		'callback', 'eigenvis(''draw'');',...
		'backgroundcolor', math_field);	
		
	uicontrol(fig_han,...
		'style', 'frame',...
		'backgroundcolor', 'b',...
		'position', [10 100 150 1]);

	uicontrol(fig_han,...
		'string', 'Help',...
		'position', [10 55 150 35],...
		'callback', 'helpwin eigenvis');

	uicontrol(fig_han,...
		'string', 'Quit',...
		'position', [10 10 150 35],...
		'callback', 'eigenvis(''end'');');
		
	eigen_axes = axes(...
		'units', 'normalized',...
		'position', [.30 0.05 .65 .9],...
		'XLimMode', 'auto',...
		'YLimMode', 'auto',...
		'visible', 'off');

	eigenvis('draw');
			
elseif strcmp(cmd,'draw')

	try
		a = str2num(cleanup(get(Tmatrix(1),'string')));
		b = str2num(cleanup(get(Tmatrix(2),'string')));
		c = str2num(cleanup(get(Tmatrix(3),'string')));
		d = str2num(cleanup(get(Tmatrix(4),'string')));
	catch
		errmesg('matrix A', 'num')
		return
	end
	
	T = [a b; c d];

	deter = det(T);

	set(det_text, 'string', ['det(A) = ' num2str(deter,8)]);

	cla;

	the_lim = max([max(abs(eig(T))) 1]);

	axis([-the_lim the_lim -the_lim the_lim]);
	axis square;

	hold on

	theta = -pi : pi/80 : pi;
	plot(cos(theta), sin(theta), 'Color', [0.4 0.4 0.4]);

	if length(save_x) == 2
		x = save_x;
	else
		x = [cos(pi/4) sin(pi/4)];
	end

	Tx = T * x' + [0;0];

	vline = line([0 x(1)], [0 x(2)], 'Erase', 'Xor', 'Color', 'r');
	Tvline = line([0 Tx(1)], [0 Tx(2)], 'Erase', 'Xor', 'Color', 'b');	
	
	hold off
		
	set(v_text, 'string', ['v = [' num2str(x(1),4) ',' num2str(x(2),4) ']']);
	set(Tv_text, 'string', ['A*v = [' num2str(Tx(1),4) ',' num2str(Tx(2),4) ']']);
	set(normTv_text, 'string', ['norm(A*v) = ' num2str(norm(Tx)) ]);
	
elseif strcmp(cmd, 'end')

	w = findobj('tag', 'eigenv_main');
	for i = 1:length(w)
		delete(w(i))
	end

	% if it's open, return to the main menu

	w = findobj('tag', 'math_main_menu');
	if length(w)
		figure(w(1));
	end

end;
