function result=exponentials(cmd);
% EXPONENTIALS
%
% You can prove, using the definition of differentiation, that the
% derivative of a^x is k*a^x for some constant k, which varies with a.
% This module lets you visually estimate the a for which k=0.
%
% 2.0/17-July-1998

% Original by Julie Mitchell
% revised by Tom Insel (spring/summer 1998)

if nargin<1,
    cmd='start';
end;

persistent freq amp fixed_x fixed_y time_line deriv_line a a_text

math_bkgnd = [0.6 0.8 1.0];
math_field = [1 1 1];

init_a = 1;
min_a = 0;
max_a = 10;

if strcmp(cmd, 'start')

	control_figures = findobj('tag', 'exponentials_main');
	if length(control_figures) == 0
		control_figures = exponentials('makemain');
	end

	figure(control_figures(1));

elseif strcmp(cmd,'makemain')

    figNumber = figure(...
        'NumberTitle', 'off', ...
        'Name', 'Exponentials', ...
		'backingstore','off',...
		'position', [20 20 600 400],...
		'resize', 'off',...
		'color', math_bkgnd,...
		'tag', 'exponentials_main');
 
	uicontrol(figNumber,...
		'style', 'text',...
		'position', [10 10 50 20],...
		'horizontalalignment', 'right',...
		'backgroundcolor', math_bkgnd,...
		'string', 'a =');
		
	a_text = uicontrol(figNumber,...
		'style', 'edit',...
		'backgroundcolor', math_field,...
		'position', [70 10 100 20],...
		'callback', 'exponentials(''readparams'')',...
		'string', num2str(init_a));

    uicontrol(figNumber, ...
        'Style','push', ...
        'String','Help', ...
        'Position',[400 10 90 25], ...
        'Callback','helpwin exponentials');
 
	uicontrol(figNumber,...
		'Style','Pushbutton', ...
        'Position',[500 10 90 25], ...
        'Callback','exponentials(''done'')',...
		'String','Quit');
				
	uicontrol(figNumber,...
		'style', 'text',...
		'string', 'f(x) = a^x',...
		'foregroundcolor', 'b',...
		'backgroundcolor', math_bkgnd,...
		'position', [200 10 80 15]);

	uicontrol(figNumber,...
		'style', 'text',...
		'string', 'f''(x)',...
		'foregroundcolor', 'r',...
		'backgroundcolor', math_bkgnd,...
		'position', [300 10 80 15]);

    axes('units', 'pixels', 'Position',[20 60 560 320]);
	hold on
	plot([0 1],[0 0],'k:');
    time_line = plot(0, 0, 'ButtonDownFcn', 'exponentials(''down'')');
	deriv_line = plot(0,0,'r', 'ButtonDownFcn', 'exponentials(''down'')');
	hold off
    axis([0 1 -1 3])
    xlabel('t')
    title('Click and drag the blue curve to change a.')
	
	exponentials('readparams')
	
	result = figNumber;
     
elseif strcmp(cmd,'down')

	set(time_line, 'erasemode', 'xor');
	set(deriv_line, 'erasemode', 'xor');

    pt=get(gca,'currentpoint');

    % find closest point on line to mouse click loc
    line_x = get(time_line, 'XData');
    line_y = get(time_line, 'YData');
    [temp,i] = min((line_x-pt(1,1)).^2 + (line_y-pt(1,2)).^2);	
    fixed_x = line_x(i);
    fixed_y = line_y(i);

    set(gcf,...
		'WindowButtonMotionFcn', 'exponentials(''move'')',...
		'WindowButtonUpFcn', 'exponentials(''up'')',...
		'pointer', 'crosshair');
	
elseif strcmp(cmd, 'readparams')

	try
		a = min(max_a, max(min_a, str2num(cleanup(get(a_text, 'string')))));
	catch
		errmesg('a','num');
		return
	end
	
	if a < 2*eps
		set(a_text, 'string', '0');
	else
		set(a_text, 'string', num2str(a));
	end
	
	exponentials('redraw')

elseif strcmp(cmd,'move') | strcmp(cmd, 'up')

    pt=get(gca,'currentpoint');
    x=pt(1,1);
    y=pt(1,2);
	
	a = min(max_a, max(min_a, y));
	
	if strcmp(cmd, 'move')
		if a < 2*eps
		    set(time_line, 'YData', [0 0], 'XData', [0 1]);
			set(deriv_line, 'YData', [0 0], 'XData', [0 0]);
			set(a_text, 'string', '0');
		else
			x = linspace(0, 1,100);
		    set(time_line, 'YData', a.^x, 'XData', x);
			set(deriv_line, 'YData', log(a)*a.^x, 'XData', x);
			set(a_text, 'string', num2str(a));
		end
	elseif strcmp(cmd, 'up')
	    set(gcf,...
			'WindowButtonMotionFcn', '',...
			'WindowButtonUpFcn', '',...
			'pointer', 'arrow');
		exponentials('redraw');
	end

elseif strcmp(cmd,'redraw')
		
	if a < 2*eps
	    set(time_line, 'YData', [0 0], 'XData', [0 1]);
		set(deriv_line, 'YData', [0 0], 'XData', [0 0]);
	else
		x = linspace(0,1,500);
	    set(time_line, 'YData', a.^x, 'XData', x, 'erasemode', 'normal');
		set(deriv_line, 'YData', log(a)*a.^x, 'XData', x, 'erasemode', 'normal');
	end
  
elseif strcmp(cmd,'done')

	w = findobj('tag', 'exponentials_main');
	for i = 1:length(w)
		delete(w(i))
	end

	% if it's open, return to the main menu

	w = findobj('tag', 'math_main_menu');
	if length(w)
		figure(w(1));
	end

end
