% FIRST ORDER ODE PLOTTER
%
% Click to plot solution curves of a first order ordinary 
% differential equation y' + f(y,t) y  = g(t). 


function r = FirstOrderODE (action, y) 

persistent h_Fig h_xmin h_xmax h_ymin h_ymax ...
   h_coeffy h_coeffconst h_ax h_gensoln 

if nargin == 0
   action = 'Init' ;
end

if nargin == 2
   t = action ;
   r = eval ( get (gcbf, 'UserData') ) ;
   return ;
end


switch (action)
   
case 'Init', 
   h_Fig = findobj ('Tag', 'FirstOrderODE') ;
   if isempty (h_Fig)
      r = CreateODEWindow ;
      h_Fig = r(1) ;
      h_xmin = r(2) ;
      h_xmax = r(3) ;
      h_ymin = r(4) ;
      h_ymax = r(5) ;
      h_coeffy = r(6) ;
      h_coeffconst = r(7) ;
      h_ax = r(8) ;
      h_gensoln = r(9) ;
	   FirstOrderODE('Update') ;
   else
      figure (h_Fig(1)) 
   end

case 'Update',
   try
      Xmin = str2num (cleanup(get (h_xmin, 'String'))) ;
      Xmax = str2num (cleanup(get (h_xmax, 'String'))) ;
   	Ymin = str2num (cleanup(get (h_ymin, 'String'))) ;
	   Ymax = str2num (cleanup(get (h_ymax, 'String'))) ;
   if Xmin >= Xmax
      XLim = get (h_ax, 'XLim') ;
      Xmin = XLim (1) ;
      Xmax = XLim (2) ;
      set (h_xmin, 'String', Xmin) ;
      set (h_xmax, 'String', Xmax) ;
   end
   if Ymin >= Ymax 
      YLim = get (h_ax, 'YLim') ;
      Ymin = YLim (1) ;
      Ymax = YLim (2) ;
      set (h_ymin, 'String', Ymin) ;
      set (h_ymax, 'String', Ymax) ;
	end    
   
   axis([Xmin, Xmax, Ymin, Ymax]);
   catch
      errordlg ('There is an error in one of the limits.') 
      error (lasterr)
	end	
   
   try 
	   s_CoeffY    = mysub(cleanup (get (h_coeffy, 'String')),'x','t');
      s_CoeffConst= mysub(cleanup (get (h_coeffconst, 'String')),'x','t') ;
		      
   
	   hold off
   	delete (get (h_ax, 'Children')) 
      xlabel('t') ;
      ylabel('y') ;
      
      axis manual
      hold on
      
      T = get (h_ax, 'XLim') ;
      Y = get (h_ax, 'YLim') ;
      
      [t, y] = meshgrid (linspace(T(1),T(2),25), ...
         linspace(Y(1),Y(2),25) ) ;
      DyDt = eval (vectorize ( [ s_CoeffConst '- y * ' s_CoeffY ] ) ) ;
      
      Normalized = sqrt(1 + DyDt.^2) ;
      dt = 1 ./ Normalized ;
      dy = DyDt  ./ Normalized ;
      
      h = quiver(t, y, dt, dy, .7) ;
      set ( h, 'Color', [.6 .6 .6]) ;
   catch
      errordlg ('There is an error in the differential equation.') 
   	error(lasterr)
   end	
   
case 'InvalidateSymSol',
   set(h_gensoln, 'String', ...
      'Click ''General Solution'' to compute y exactly.') ;
   
case 'SolutionCurve',
      
   p = get (h_ax, 'CurrentPoint') ;
   
   t0=p(1,1);
   y0=p(1,2);
   
   try 
	   s_CoeffY    = mysub(cleanup (get (h_coeffy, 'String')),'x','t');
      s_CoeffConst= mysub(cleanup (get (h_coeffconst, 'String')),'x','t') ;
      
      G = vectorize ( [s_CoeffConst '- y * ' s_CoeffY ] ) ;

      set (h_Fig, 'UserData', G ) ;
      
      options = odeset('RelTol',.05,'AbsTol',.05) ;
      T = get (h_ax, 'XLim') ;
      time = T(2) - T(1) ;
      [t,y] = ode45 ('FirstOrderODE', ...
         [t0,t0+time], y0, options);
      h = plot (t,y) ;          
      set (h, 'Color', [.6 .5 1]) ;      
      
      % Go backwards in time by replacing t by 2*t0-t
      % and doing it again.  This difficult looking
      % parsing is necessary because replacing all 
      % instances of t with 2*t0-t doesn't work because
      % it would mangle functions like tan(t).  
      
      H = mysub (G, 't', ['(2*' num2str(t0) '-t)']) ;
      H = ['-(' H ')'] ;         
      
      set (h_Fig, 'Userdata', H) ; 
      
      [t,y] = ode45 ('FirstOrderODE', ...
         [t0,t0+time], y0, options);
      
      h = plot (2*t0-t,y) ;          
      set (h, 'Color', 'b') ;      
      
   catch
      errordlg ('There is an error in the differential equation.') 
      error (lasterr)
	end	
      
case 'ZoomOut',
   T = get (h_ax, 'XLim') ;
   newT = T + [-1,1]*(T(2)-T(1))/3 ;
   newT = [newT(1), newT(2)] ;
   set (h_ax, 'XLim', newT ) ;
   Y = get (h_ax, 'YLim') ;
   newY = Y + [-1,1]*(Y(2)-Y(1))/3 ;
   newY = [newY(1), newY(2)] ;
   set (h_ax, 'YLim', newY ) ;
   
   set (h_xmin, 'String', num2str(newT(1)) ) ;
   set (h_xmax, 'String', num2str(newT(2)) ) ;
   set (h_ymin, 'String', num2str(newY(1)) ) ;
   set (h_ymax, 'String', num2str(newY(2)) ) ;
  
   FirstOrderODE ('Update') ;   
   
case 'ZoomIn',
   T = get (h_ax, 'XLim') ;
   newT = T + [1,-1]*(T(2)-T(1))/5 ;
   set (h_ax, 'XLim', newT ) ;
   Y = get (h_ax, 'YLim') ;
   newY = Y + [1,-1]*(Y(2)-Y(1))/5 ;
   set (h_ax, 'YLim', newY ) ;
   
   set (h_xmin, 'String', num2str(newT(1)) ) ;
   set (h_xmax, 'String', num2str(newT(2)) ) ;
   set (h_ymin, 'String', num2str(newY(1)) ) ;
   set (h_ymax, 'String', num2str(newY(2)) ) ;
  
   FirstOrderODE ('Update') ;   

   
case 'Library',
   msgbox ('Library not implimented.') ;
   
case 'GeneralSolution',
	s_CoeffY    = mysub(cleanup (get (h_coeffy, 'String')),'x','t');
   s_CoeffConst= mysub(cleanup (get (h_coeffconst, 'String')),'x','t') ;
   
   Eqn = ['Dy = ' s_CoeffConst '- y * (' s_CoeffY ')'] ;

   SymSoln = dsolve (Eqn) ;
   if isempty (SymSoln) 
      warndlg ('Maple was unable to find a symbolic solution to the given ODE.', ...
         'No Solution') ;
     	set (h_gensoln, 'String', 'Click ''General Solution'' to compute y exactly.') ;    
   else
      set (h_gensoln, 'String', char(SymSoln) ) ;
   end 


case 'Help',
   helpwin ('FirstOrderODE') ;
end

r='';


function r = CreateODEWindow ()
col = [.6 .8 1];

h_fig = figure('Color',col, ...
	'Name','FIRST ORDER ODE PLOTTER', ...
	'NumberTitle','off', ...
	'Position',[0 0 635 400], ...
	'Resize','off', ...
	'Tag','FirstOrderODE', ...
	'UserData','sin(t)- y .* 1', ...
   'WindowButtonDownFcn','FirstOrderODE(''SolutionCurve'');', ...
   'Menubar', 'none');




h_ax = axes('Parent',h_fig, ...
   'units','pixels',...
	'Box','on', ...
	'Position',[175 70 440 320]);

uicontrol (h_fig, 'style', 'text',...
   'string', 't range',...
   'position',[10 360 40 20],...
   'backgroundcolor',col) ;

h_xmin = uicontrol('Parent',h_fig, ...
	'BackgroundColor',[1 1 1], ...
	'Callback','FirstOrderODE(''Update'');', ...
	'Position',[60 360 35 20], ...
	'String','-5', ...
   'Style','edit');

h_xmax = uicontrol('Parent',h_fig, ...
	'BackgroundColor',[1 1 1], ...
	'Callback','FirstOrderODE(''Update'');', ...
	'Position',[95 360 35 20], ...
	'String','5', ...
	'Style','edit');

uicontrol (h_fig, 'style', 'text',...
   'string', 'y range',...
   'position',[10 330 40 20],...
   'backgroundcolor',col) ;

h_ymin = uicontrol('Parent',h_fig, ...
	'BackgroundColor',[1 1 1], ...
	'Callback','FirstOrderODE(''Update'');', ...
	'Position',[60 330 35 20], ...
	'String','-5', ...
	'Style','edit') ;

h_ymax = uicontrol('Parent',h_fig, ...
	'BackgroundColor',[1 1 1], ...
	'Callback','FirstOrderODE(''Update'');', ...
	'Position',[95 330 35 20], ...
	'String','5', ...
	'Style','edit') ;




uicontrol('Parent',h_fig, ...
	'BackgroundColor',col, ...
	'Position',[15 260 100 50], ...
	'String','Click mouse to draw solution curves.', ...
	'Style','text');

if havesymbolic	
uicontrol('Parent',h_fig, ...
	'Callback','FirstOrderODE (''GeneralSolution'');', ...
	'Position',[10 235 110 30], ...
   'String','General Solution');
end
   
uicontrol('Parent',h_fig, ...
	'Callback','FirstOrderODE (''ZoomIn'');', ...
	'Position',[10 200 110 30], ...
	'String','Zoom In');

uicontrol('Parent',h_fig, ...
	'Callback','FirstOrderODE (''ZoomOut'');', ...
	'Position',[10 165 110 30], ...
   'String','Zoom Out');

%uicontrol('Parent',h_fig, ...
%	'Callback','FirstOrderODE (''Library'');', ...
%	'ListboxTop',0, ...
%	'Position',[10 130 110 30], ...
%   'String','Library');

uicontrol('Parent',h_fig, ...
	'Callback','FirstOrderODE (''Help'');', ...
	'Position',[10 95 110 30], ...
   'String','Help');

uicontrol('Parent',h_fig, ...
	'Callback','close (gcbf)', ...
	'Position',[10 60 110 30], ...
	'String','Quit');

uicontrol('Parent',h_fig, ...
	'BackgroundColor',col, ...
	'Position',[20 10 50 20], ...
	'String','dy/dt  + ', ...
	'Style','text');

uicontrol('Parent',h_fig, ...
	'BackgroundColor',col, ...
	'Position',[130 10 40 20], ...
	'String','  * y  =   ', ...
	'Style','text');

h_coeffy = uicontrol('Parent',h_fig, ...
	'BackgroundColor',[1 1 1], ...
	'Position',[80 10 54 20], ...
   'String','1', ...
   'horizontalalignment', 'right', ...
	'Style','edit', ...
   'Tag','CoeffY', ...
   'CallBack', ...
   'FirstOrderODE(''InvalidateSymSol'');FirstOrderODE(''Update'');');

h_coeffconst = uicontrol('Parent',h_fig, ...
	'BackgroundColor',[1 1 1], ...
   'Position',[170 10 54 20], ...
   'horizontalalignment', 'left', ...
	'String','sin(t)', ...
	'Style','edit', ...
   'Tag','CoeffConst', ...
   'CallBack', ...
   'FirstOrderODE(''InvalidateSymSol'');FirstOrderODE(''Update'');');


   if havesymbolic
uicontrol('Parent',h_fig, ...
	'BackgroundColor',col, ...
	'Position',[300 10 20 20], ...
	'String','y = ', ...
	'Style','text');

	end
h_gensoln = uicontrol('Parent',h_fig, ...
	'BackgroundColor',[1 1 1], ...
	'HorizontalAlignment','left', ...
	'ListboxTop',0, ...
	'Position',[340 10 270 20], ...
	'String','Click ''General Solution'' to compute y exactly.', ...
	'Style','edit', ...
	'Tag','GeneralSolutionBox');
if ~havesymbolic
	set(h_gensoln,'visible','off')
end

	

r = [h_fig h_xmin h_xmax h_ymin h_ymax ...
     h_coeffy h_coeffconst h_ax h_gensoln] ;
   


function H = mysub(G, s, t) 
H = '' ;
for i=1:length(G) 
   if (G(i) ~= s)
      H = [H G(i)] ;
   else   
      if i>1 & (('A'<=G(i-1) & G(i-1)<='Z')|...
            ('a'<=G(i-1) & G(i-1)<='z'))
         H = [H G(i)] ;
      elseif i<length(G) & ...
            (('A'<=G(i+1) & G(i+1)<='Z')|...
            ('a'<=G(i+1) & G(i+1)<='z'))
         H = [H G(i)] ;   
      else
         H = [H t] ;
      end
   end
end
