function result = geom2d(cmd, arg)
% 2-D GEOMETRY
%
% This is a module for graphing points and lines in two dimensions.
% 
% To add a point to the graph, specify its coordinates in the first two
% boxes and press the "Add Point" button.  To add a line segment to the
% graph, specify the two end points and press the "Add Line Segment" button.
% 
% To delete a point or segment, press the "Delete" button.  Then, click
% on the object you wish to delete.
% 
% Alternatively, you can check the "Click to Choose Points" box.  Then, to
% add a line, press the "Add Point" button and click on the graph. To add a
% line segment, press the "Add Line Segment" and click twice on the graph.
% 
% The black list on the left will summarize the items currently in the graph.

% geom2d
%
% Graph lines and planes in R^2, written for William to use with
% Math 54M, and as warmup for a module to graph points, lines
% and planes in R^3.
%
% REQUIRES: cleanup.m
%
% HISTORY:
% - Original by Tom Insel (February 1998).
% - Based loosely on geo3d.m/geom3d.m which was written by Alex Gottlieb
%   and Dave Faris (Summer 1996) and improved by Julie Mitchell.
%
% TODO:
% - widen the listbox
% - selecting a line in the listbox should highlight the vector or point, and
%   clicking on it when delete is active should delete it.
%
% TESTED: MATLAB 5.2 for Macintosh

% colors

math_bkgnd = [0.6 0.8 1.0];
math_field = [1 1 1]; 

if ~nargin, cmd = 'initialize'; end

if strcmp(cmd, 'initialize')

	geom2d('regraph')

	% Bring the control figure to the front.

	control_figures = findobj('tag', 'geomd2d_main');
	if length(control_figures) == 0
		control_figures = geom2d('makemain');
	end

	figure(control_figures(1));

elseif strcmp(cmd, 'makemain')

	main_figure = figure(...
		'position', [10 10 600 400], ...
		'name', '2-D Geometry', ...
		'numbertitle', 'off',...
		'resize', 'off',...
		'Tag', 'geomd2d_main');

	uicontrol(main_figure,...
		'style', 'frame',...
		'position', [-5 -5 205 410],...
		'backgroundcolor', math_bkgnd);
		
	uicontrol(main_figure,...
		'position', [10 370 60 20],...
		'string', 'Add Point',...
		'callback', 'geom2d(''addpoint'');');		
		
	point_x = uicontrol(main_figure,...
		'style', 'edit',...
		'position', [80 370 55 20],...
		'backgroundcolor', math_field,...
		'string', '3');
		
	point_y = uicontrol(main_figure,...
		'style', 'edit',...
		'position', [135 370 55 20],...
		'backgroundcolor', math_field,...
		'string', '4');
		
	uicontrol(main_figure,...
		'position', [10 360 180 1],...
		'style', 'frame',...
		'backgroundcolor', 'k');
		
	uicontrol(main_figure,...
		'position', [10 330 100 20],...
		'string', 'Add Line Segment',...
		'callback', 'geom2d(''addline'');');

	seg_label1 = uicontrol(main_figure,...
		'style', 'text',...
		'string', 'from',...
		'position', [10 305 25 20],...
		'horizontalalign', 'right',...
		'backgroundcolor', math_bkgnd);

	seg_x0 = uicontrol(main_figure,...
		'style', 'edit',...
		'position', [45 305 30 20],...
		'backgroundcolor', math_field,...
		'string', '0');
		
	seg_y0 = uicontrol(main_figure,...
		'style', 'edit',...
		'position', [75 305 30 20],...
		'backgroundcolor', math_field,...
		'string', '0');
		
	seg_label2 = uicontrol(main_figure,...
		'style', 'text',...
		'string', 'to',...
		'position', [110 305 12 20],...
		'horizontalalign', 'right',...
		'backgroundcolor', math_bkgnd);
		
	seg_x1 = uicontrol(main_figure,...
		'style', 'edit',...
		'position', [130 305 30 20],...
		'backgroundcolor', math_field,...
		'string', '3');
		
	seg_y1 = uicontrol(main_figure,...
		'style', 'edit',...
		'position', [160 305 30 20],...
		'backgroundcolor', math_field,...
		'string', '3');

	uicontrol(main_figure,...
		'position', [10 295 180 1],...
		'style', 'frame',...
		'backgroundcolor', 'k');

	click_check = uicontrol(main_figure,...
		'position', [10 265 180 20],...
		'style', 'popupmenu',...
		'string', 'Choose Points by Coordinates|Choose Points with Mouse',...
		'value', 1,...
		'callback', 'geom2d(''fixclick'');');

	uicontrol(main_figure,...
		'position', [10 240 85 20],...
		'string', 'Delete...',...
		'callback', 'geom2d(''deleteon'');');

	uicontrol(main_figure,...
		'position', [105 240 85 20],...
		'string', 'Delete All',...
		'callback', 'cla,geom2d(''deleteoff'')');

	click_description = uicontrol(main_figure,...
		'position', [10 55 180 175],...
		'style', 'listbox',...
		'callback', 'geom2d(''makelist'');');

	uicontrol(main_figure,...
		'string', 'Help',...
		'position', [105 10 85 35],...
		'callback', 'helpwin geom2d');

	uicontrol(main_figure,...
		'string', 'Quit',...
		'position', [10 10 85 35],...
		'callback', 'geom2d(''end'');');

	main_axis = axes('Position', [.41 .075 .56 .86]);
	xlabel('x')
	ylabel('y')
	axis('equal')
	
	set(main_figure, 'userdata',...
		[main_axis; point_x; point_y; seg_x0; seg_y0; seg_x1; seg_y1;...
		 click_description; click_check; seg_label1; seg_label2; 0]);
	
	% the 0 is for deletemode
	
	result = main_figure;
	
elseif strcmp(cmd, 'addpoint') | strcmp(cmd, 'addline') | strcmp(cmd, 'deleteon')...
		| strcmp(cmd, 'clickme') | strcmp(cmd, 'makelist') | strcmp(cmd, 'fixclick')...
		| strcmp(cmd, 'deleteoff')

	main_figures = findobj('tag', 'geomd2d_main');
	if length(main_figures) == 0
		main_figures = geom2d('makemain');
	end
		
	main_figure = main_figures(1);
		
	c = get(main_figure, 'userdata');
	main_axis = c(1);
	point_x = c(2);
	point_y = c(3);
	seg_x0 = c(4);
	seg_y0 = c(5);
	seg_x1 = c(6);
	seg_y1 = c(7);
	click_description = c(8);
	click_check = c(9);
	seg_label1 = c(10);
	seg_label2 = c(11);
	deletemode = c(12); % this gets set later, so be sure to change below if renumbered

	err_field=0;
	err_type=0;
	
	if strcmp(cmd, 'addpoint')
		figure(main_figure)
		
		if get(click_check, 'value') == 2
			title('Click to Choose a Point');
			[x y] = ginput(1);
			title('');
		else		
			x_string = cleanup(get(point_x, 'string'));
			y_string = cleanup(get(point_y, 'string'));
			eval([x_string ';'], 'err_field=''x''; err_type=''num'';');
			eval([y_string ';'], 'err_field=''y''; err_type=''num'';');		
		
			if err_field
				errmesg(err_field, err_type);
				error(lasterr);
				return
			end
		
			x = str2num(x_string);
			y = str2num(y_string);
		end
		
		figure(main_figure)
		hold on
		the_plot = plot(x,y,'r.');
		set(the_plot,...
			'markersize', 15,...
			'tag', ['Point: [' num2str(x,4) ',' num2str(y,4) ']'],...
			'ButtonDownFcn', 'geom2d(''clickme'');');		
		hold off
		
	elseif strcmp(cmd, 'addline')
		figure(main_figure)
		hold on

		if get(click_check, 'value') == 2		
			title('Click at the Beginning of the Line Segment')
			[x y] = ginput(1);
			axis('manual')
			temp_plot = plot(x,y,'x');
			title('Click at the End of the Line Segment')
			drawnow
			[x(2) y(2)] = ginput(1);
			axis('auto')
			title('')
			delete(temp_plot)
		else		
			x0_string = cleanup(get(seg_x0, 'string'));
			x1_string = cleanup(get(seg_x1, 'string'));
			y0_string = cleanup(get(seg_y0, 'string'));
			y1_string = cleanup(get(seg_y1, 'string'));
			eval([x0_string ';'], 'err_field=''x0''; err_type=''num'';');
			eval([x1_string ';'], 'err_field=''x1''; err_type=''num'';');
			eval([y0_string ';'], 'err_field=''y0''; err_type=''num'';');
			eval([y1_string ';'], 'err_field=''y1''; err_type=''num'';');

			if err_field
				errmesg(err_field, err_type);
				error(lasterr);
				return
			end

			x = [str2num(x0_string) str2num(x1_string)];
			y = [str2num(y0_string) str2num(y1_string)];

		end

		% figure(main_figure)
		the_plot = plot(x,y);
		set(the_plot,...
			'tag', ['Segment: [' num2str(x(1),3) ',' num2str(y(1),3) ...
					'] to ['  num2str(x(2),3) ',' num2str(y(2),3) ']'],...
			'ButtonDownFcn', 'geom2d(''clickme'');')
		hold off
		
	elseif strcmp(cmd, 'clickme')
	
		if deletemode
			delete(gco)
			title('')
		end
	
	elseif strcmp(cmd, 'makelist')
	
		% list current objects in click_description
	
		s = 0;
		
		h = findobj(main_axis);
		for i = 1:length(h)
			if get(h(i), 'type') == 'line'
				s = str2mat(s, get(h(i), 'tag'));
			end
		end
		
		set(click_description, 'string', s(2:size(s,1),:));
	
	elseif strcmp(cmd, 'fixclick')
	
		if get(click_check, 'value') == 2
			set(point_x, 'visible', 'off');
			set(point_y, 'visible', 'off');
			set(seg_x0, 'visible', 'off');
			set(seg_y0, 'visible', 'off');
			set(seg_x1, 'visible', 'off');
			set(seg_y1, 'visible', 'off');
			set(seg_label1, 'visible', 'off');
			set(seg_label2, 'visible', 'off');
		else
			set(point_x, 'visible', 'on');
			set(point_y, 'visible', 'on');
			set(seg_x0, 'visible', 'on');
			set(seg_y0, 'visible', 'on');
			set(seg_x1, 'visible', 'on');
			set(seg_y1, 'visible', 'on');
			set(seg_label1, 'visible', 'on');
			set(seg_label2, 'visible', 'on');
		end
	
	end	

	if ~strcmp(cmd, 'makelist') % clean this up
		geom2d('makelist')
		figure(main_figure)
		
		if strcmp(cmd, 'deleteon') & ~deletemode
			set(main_figure, 'Pointer', 'cross') % or crosshair or circle?
			deletemode = 1;
			title('Click on a Point or Segment to Delete It.')
		else
			set(main_figure, 'Pointer', 'arrow');
			deletemode = 0;
			title('')
		end
		
		c(12) = deletemode;
		set(main_figure, 'userdata', c)
	end

elseif strcmp(cmd, 'end')

	w = findobj('tag', 'geomd2d_main');	
	for i = 1:length(w)
		delete(w(i))
	end

end
