function result = grafsurf(cmd, arg)
% SURFACES
%
% This module is for graphing parametrized surfaces.
%
% *** This text is outdated. ***
% 
% Use this window to specify a surface in the form
%     [ x(u,v) y(u,v) z(u,v) ]
% and the domain in u and v over which it will be graphed. If you
% want to draw a parametric curve to be plotted on the surface,
% enter it in the u(t) and v(t) fields and make sure the "Graph
% Curve?" box is checked. Finally, use the UPDATE GRAPH button
% plot the surface.
% 
% The window shows the rectangle in the uv-plane over which you are
% graphing the function.  To place a red, green, or blue marker on
% the domain, select the appropriate ADD button.  When the
% cursor changes to a crosshair,click where you would like the
% marker to go. It will also be drawn in the range window.

% Original by Alex Gottlieb, update by Tom Insel (spring/summer 1998)
% adapted from grafsurf.m and parasurf.m
%
% needs error detection, etc.
%
% --- Markers need fixing
% --- Curve in xyz-space


persistent curve_option x_of_uv lo_u lo_v hi_u hi_v u_of_t v_of_t lo_t hi_t y_of_uv z_of_uv
persistent domain_axes
persistent red_dot_uv white_dot_uv blue_dot_uv
persistent red_dot_xyz white_dot_xyz blue_dot_xyz
persistent red_xyz white_xyz blue_xyz

% colors

math_bkgnd = [0.6 0.8 1.0];
math_field = [1 1 1];

if ~nargin, cmd = 'start'; end

if strcmp(cmd, 'start')

	grafsurf('regraph');

	control_figures = findobj('tag', 'grafsurf_control');
	if length(control_figures) == 0
		control_figures = grafsurf('makecontrol');
	end

	figure(control_figures(1));

elseif strcmp(cmd, 'regraph') | strcmp(cmd, 'marker')

	control_figures = findobj('tag', 'grafsurf_control');
	if length(control_figures) == 0
		control_figures = grafsurf('makecontrol');
	end

	control_figure = control_figures(1);

	% These are for error detection. After we get text from a field, we try to
	% eval() it, setting err_field & err_type if this fails. If an error occurs,
	% call errmesg and bail.  It is important not to go on to the real eval()
	% or str2num() lest we go on to plot in the help window.

	err_field=0;
	err_type=0;

	if strcmp(cmd, 'regraph')
		
		lo_u_string = cleanup(get(lo_u, 'string'));
		eval([lo_u_string ';'], 'err_field=''low u value''; err_type=''num'';');
		
		hi_u_string = cleanup(get(hi_u, 'string'));
		eval([hi_u_string ';'], 'err_field=''high u value''; err_type=''num'';');
		
		lo_v_string = cleanup(get(lo_v, 'string'));
		eval([lo_v_string ';'], 'err_field=''low v value''; err_type=''num'';');
		
		hi_v_string = cleanup(get(hi_v, 'string'));
		eval([hi_v_string ';'], 'err_field=''high v value''; err_type=''num'';');

		if err_field
			errmesg(err_field, err_type);
			error(lasterr);
		end

		u0 = str2num(lo_u_string);
		u1 = str2num(hi_u_string);
		v0 = str2num(lo_v_string);
		v1 = str2num(hi_v_string);

		axes(domain_axes);
		axis([u0 u1 v0 v1]);
		zoom on
		grid on
		hold on
		xlabel('u')
		ylabel('v')	

		[u v] = meshgrid( linspace(u0,u1,30), linspace(v0,v1,30));
				
		red_dot_uv = 0;
		white_dot_uv = 0;
		blue_dot_uv = 0;
		purple_dot_uv = 0;
		
		if get(curve_option, 'value')
			lo_t_string = cleanup(get(lo_t, 'string'));
			eval([lo_t_string ';'], 'err_field=''low t value''; err_type=''num'';');
	
			hi_t_string = cleanup(get(hi_t, 'string'));
			eval([hi_t_string ';'], 'err_field=''high t value''; err_type=''num'';');
	
			if err_field
				errmesg(err_field, err_type);
				error(lasterr);
			end
	
			t0 = str2num(lo_t_string);
			t1 = str2num(hi_t_string);
			t = linspace(t0, t1, 300);
			
			u_of_t_string = vectorize(cleanup(get(u_of_t, 'string')));
			eval([u_of_t_string ';'], 'err_field=''u(t)''; err_type=''fun'';');
	
			v_of_t_string = vectorize(cleanup(get(v_of_t, 'string')));
			eval([v_of_t_string ';'], 'err_field=''v(t)''; err_type=''fun'';');
	
			if err_field
				errmesg(err_field, err_type);
				error(lasterr);
			end
	
			u_curve = eval(u_of_t_string);
			if length(u_curve) == 1, u_curve = u_curve + 0*t; end
	
			v_curve = eval(v_of_t_string);
			if length(v_curve) == 1, v_curve = v_curve + 0*t; end
	
			plot(u_curve, v_curve);
		end % curve_option

		hold off

		range_figures = findobj('tag', 'grafsurf_range');
		if length(range_figures) == 0 
			range_figures = grafsurf('makerange');
		end
	
		figure(range_figures(1))
		
		cla
				
		red_dot_xyz = 0;
		white_dot_xyz = 0;
		blue_dot_xyz = 0;

		set(red_xyz, 'string', ' ');
		set(white_xyz, 'string', ' ');
		set(blue_xyz, 'string', ' ');

		hold on

		x_string = vectorize(cleanup(get(x_of_uv, 'string')));
		eval([x_string ';'], 'err_field=''x(u,v)''; err_type=''fun'';');

		y_string = vectorize(cleanup(get(y_of_uv, 'string')));
		eval([y_string ';'], 'err_field=''y(u,v)''; err_type=''fun'';');

		z_string = vectorize(cleanup(get(z_of_uv, 'string')));
		eval([z_string ';'], 'err_field=''z(u,v)''; err_type=''fun'';');

		if err_field
			errmesg(err_field, err_type);
			error(lasterr);
		end
				
		x = eval(x_string);
		y = eval(y_string);
		z = eval(z_string);
		if length(x) == 1, x = 0*u; end
		if length(y) == 1, y = 0*u; end
		if length(z) == 1, z = 0*u; end

		wave = surf(x,y,z);
		
%		wave = surfl(x,y,z);
%		shading interp  
%		colormap(copper)		
		
		set(wave,...
			'facelighting', 'gouraud',...
			'backfacelighting', 'lit',...
			'DiffuseStrength', 1.0,...
			'SpecularStrength',0.9,...
			'AmbientStrength', 0.5,...
			'edgecolor', [0 0 0],...
			'SpecularColorReflectance',0.1);

%		zoom off
%		grid on		
		axis square
		axis vis3d
		view(3)
%		view(52.5, 30); % restore our standard view
		rotate3d ON % caps disable feedback window

		xlabel('x')
		ylabel('y')
		zlabel('z')
		title('Click and drag to rotate the surface.')
						   
		if get(curve_option, 'value') == 1,
			utemp = u;
			vtemp = v;
			u = u_curve;
			v = v_curve;
			x = eval(x_string);
			y = eval(y_string);
			z = eval(z_string);
			if length(x) == 1, x = x + 0*u_curve; end
			if length(y) == 1, y = y + 0*u_curve; end
			if length(z) == 1, z = z + 0*u_curve; end
			plot3(x, y, z, 'linestyle', '.', 'markersize', 11)
			u = utemp;
			v = vtemp;
		end
		
		hold off

		set(red_xyz, 'string', ' ');
		set(white_xyz, 'string', ' ');
		set(blue_xyz, 'string', ' ');

	else % place markers
	
		figure(control_figure)
		axes(domain_axes)
		
		[u v] = ginput(1);
		hold on
		if arg == 'r'
			if red_dot_uv, delete(red_dot_uv), end
			red_dot_uv = plot( u, v, 'r.', 'markersize', 20);
		elseif arg == 'g'
			if white_dot_uv, delete(white_dot_uv), end
			white_dot_uv = plot( u, v, 'g.', 'markersize', 20);
		elseif arg == 'b'
			if blue_dot_uv, delete(blue_dot_uv), end
			blue_dot_uv = plot( u, v, 'b.', 'markersize', 20);
		end
		hold off
		
		range_figures = findobj('tag', 'grafsurf_range');
		if length(range_figures) == 0 
			range_figures = grafsurf('makerange');
			if red_dot_uv & arg ~= 'r', delete(red_dot_uv), red_dot_uv = 0; end
			if white_dot_uv & arg ~= 'g', delete(white_dot_uv), white_dot_uv = 0; end
			if blue_dot_uv & arg ~= 'b', delete(blue_dot_uv), blue_dot_uv = 0; end
			grafsurf('regraph');
		end
	
		figure(range_figures(1))
		hold on
		
		x = eval( vectorize( cleanup(get( x_of_uv, 'string'))));
		y = eval( vectorize( cleanup(get( y_of_uv, 'string'))));
		z = eval( vectorize( cleanup(get( z_of_uv, 'string'))));
		
		if arg == 'r',
			if red_dot_xyz, delete(red_dot_xyz), end
			red_dot_xyz = plot3( x, y, z, 'r.', 'markersize', 20);
			set(red_xyz, 'string',...
				[ '[' num2str(x,3) ' ' num2str(y,3) ' ' num2str(z,3) ']']);
		elseif arg == 'g',
			if white_dot_xyz, delete(white_dot_xyz), end
			white_dot_xyz = plot3( x, y, z, 'g.', 'markersize', 20);
			set(white_xyz, 'string',...
				[ '[' num2str(x,3) ' ' num2str(y,3) ' ' num2str(z,3) ']']);
		elseif arg == 'b',
			if blue_dot_xyz, delete(blue_dot_xyz), end
			blue_dot_xyz = plot3( x, y, z, 'b.', 'markersize', 20);
			set(blue_xyz, 'string',...
				[ '[' num2str(x,3) ' ' num2str(y,3) ' ' num2str(z,3) ']']);
		end

	end

elseif strcmp(cmd, 'makerange')

	result = figure(...
		'Position', [390 170 400 400],...
		'Name', 'Surfaces: Range',...
		'NumberTitle', 'off',...
		'Tag', 'grafsurf_range');
	
elseif strcmp(cmd, 'makecontrol')

	control_figure = figure(...
		'position', [100 100 620 350], ...
		'name', 'Surfaces: Controls', ...
		'numbertitle', 'off',...
		'resize', 'off',...
		'Color', math_bkgnd,...
		'Tag', 'grafsurf_control');
	
	uicontrol(control_figure,...
		'style', 'frame',...
		'position', [310 10 1 330],...
		'backgroundcolor', 'black');

	% The domain controls

	uicontrol(control_figure,...
		'style', 'text',...
		'position', [320 320 50 20],...
		'string', 'DOMAIN',...
		'horizontalalignment', 'left',...
		'backgroundcolor', math_bkgnd);

	lo_u = uicontrol(control_figure,...
		'style', 'edit',...
		'position', [380 320 50 20],...
		'string', '-4',...
		'backgroundcolor', math_field);
		
	if strcmp(computer,'MAC2')
		domain_string = ' u ';
	else
		domain_string = '<= u <=';
	end
		
	uicontrol(control_figure,...
		'style', 'text',...
		'position', [440 320 30 20],...
		'string', domain_string,...
		'horizontalalignment', 'left',...
		'backgroundcolor', math_bkgnd);
	   
	hi_u = uicontrol(control_figure,...
		'style', 'edit',...
		'position', [480 320 50 20],...
		'string', '4',...
		'backgroundcolor', math_field);

	lo_v = uicontrol(control_figure,...
		'style', 'edit',...
		'position', [380 295 50 20],...
		'string', '-4',...
		'backgroundcolor', math_field);
		
	if strcmp(computer,'MAC2')
		domain_string = ' v ';
	else
		domain_string = '<= v <=';
	end
		
	uicontrol(control_figure,...
		'style', 'text',...
		'position', [440 295 30 20],...
		'string', domain_string,...
		'horizontalalignment', 'left',...
		'backgroundcolor', math_bkgnd);
	   
	hi_v = uicontrol(control_figure,...
		'style', 'edit',...
		'position', [480 295 50 20],...
		'string', '4',...
		'backgroundcolor', math_field);
		
	domain_axes = axes('position', [0.56, 0.1, 0.4, 0.7]);
	axis square
	
	% surface controls

	uicontrol(control_figure,...
		'style', 'text',...
		'position', [10 320 200 20],...
		'string', 'SURFACE',...
		'horizontalalignment', 'left',...
		'backgroundcolor', math_bkgnd);

	uicontrol(control_figure,...
		'style', 'text',...
		'position', [70 320 45 20],...
		'string', 'x(u,v) =',...
		'horizontalalignment', 'right',...
		'backgroundcolor', math_bkgnd);

	x_of_uv = uicontrol(control_figure,...
		'style', 'edit',...
		'position', [120 320 180 20],...
		'string', 'u',...
		'backgroundcolor', math_field);

	uicontrol(control_figure,...
		'style', 'text',...
		'position', [70 295 45 20],...
		'string', 'y(u,v) =',...
		'horizontalalignment', 'right',...
		'backgroundcolor', math_bkgnd);

	y_of_uv = uicontrol(control_figure,...
		'style', 'edit',...
		'position', [120 295 180 20],...
		'string', 'v',...
		'backgroundcolor', math_field);

	uicontrol(control_figure,...
		'style', 'text',...
		'position', [70 270 45 20],...
		'string', 'z(u,v) =',...
		'horizontalalignment', 'right',...
		'backgroundcolor', math_bkgnd);

	z_of_uv = uicontrol(control_figure,...
		'style', 'edit',...
		'position', [120 270 180 20],...
		'string', 'u*v',...
		'backgroundcolor', math_field);
		
	uicontrol(control_figure,...
		'style', 'frame',...
		'position', [10 260 290 1]);

    % curve

	uicontrol(control_figure,...
		'style', 'text',...
		'position', [10 230 100 20],...
		'string', 'CURVE',...
		'horizontalalignment', 'left',...
		'backgroundcolor', math_bkgnd);
		
	uicontrol(control_figure,...
		'style', 'text',...
		'string', 'u(t) =',...
		'position', [60 230 50 20],...
		'horizontalalignment', 'right',...
		'backgroundcolor', math_bkgnd);

	u_of_t = uicontrol(control_figure,...
		'style', 'edit',...
		'string', 'sin(t)',...
		'position', [120 230 180 20],...
		'backgroundcolor', math_field);
	   
	uicontrol(control_figure,...
		'style', 'text', ...
		'string', 'v(t) =',...
		'position', [60 205 50 20],...
		'horizontalalignment', 'right',...
		'backgroundcolor', math_bkgnd);
	   
	v_of_t = uicontrol(control_figure,...
		'style', 'edit',...
		'string', 'cos(t)',...
		'position', [120 205 180 20],...
	    'backgroundcolor', math_field);

	lo_t = uicontrol(control_figure,...
		'style', 'edit',...
		'position', [150 180 50 20],...
		'string', '0',...
		'backgroundcolor', math_field);
	   
	if strcmp(computer,'MAC2')
		domain_string = ' t ';
	else
		domain_string = '<= t <=';
	end
		
	uicontrol(control_figure,...
		'style', 'text',...
		'position', [210 180 30 20],...
	    'string', domain_string, ...
		'horizontalalignment', 'center',...
		'backgroundcolor', math_bkgnd);
		
	hi_t = uicontrol(control_figure,...
		'style', 'edit',...
		'position', [250 180 50 20],...
		'string', '2*pi',...
		'backgroundcolor', math_field);
		
	curve_option = uicontrol(control_figure,...
		'style', 'checkbox',...
		'position', [20 180 90 20],...
		'string', 'Graph Curve?',...
		'backgroundcolor', math_bkgnd);
		
	uicontrol(control_figure,...
		'style', 'frame',...
		'position', [10 170 290 1]);
		
	% markers
	
	uicontrol(control_figure,...
		'style', 'text',...
		'position', [10 140 100 20],...
		'string', 'MARKERS',...
		'horizontalalignment', 'left',...
		'backgroundcolor', math_bkgnd);

	uicontrol(control_figure,...
		'string', 'Add Red',...
		'position', [20 115 60 20],...
		'callback', 'grafsurf(''marker'',''r'');');

	uicontrol(control_figure,...
		'string', 'Add Green',...
		'position', [20 90 60 20],...
		'callback', 'grafsurf(''marker'',''g'');');
		
	uicontrol(control_figure,...
		'string', 'Add Blue',...
		'position', [20 65 60 20],...
		'callback', 'grafsurf(''marker'',''b'');');
	
	red_xyz = uicontrol(control_figure,...
		'style', 'text',...
		'backgroundcolor', math_bkgnd,...
		'position', [90 115 200 20],...
		'foregroundcolor', 'r');
		
	white_xyz = uicontrol(control_figure,...
		'style', 'text',...
		'backgroundcolor', math_bkgnd,...
		'position', [90 90 200 20],...
		'foregroundcolor', [0 .7 0]);

	blue_xyz = uicontrol(control_figure,...
		'style', 'text',...
		'position', [90 65 200 20],...
		'backgroundcolor', math_bkgnd,...
		'foregroundcolor', 'b');

	uicontrol(control_figure,...
		'style', 'frame',...
		'position', [10 55 290 1]);
		
	% buttons

	uicontrol(control_figure,...
		'string', 'Update Graph',...
		'position', [10 10 110 35],...
		'callback', 'grafsurf(''regraph'');');

	uicontrol(control_figure,...
		'string', 'Help',...
		'position', [130 10 80 35],...
		'callback', 'helpwin grafsurf');

	uicontrol(control_figure,...
		'string', 'Quit',...
		'position', [220 10 80 35],...
		'callback', 'grafsurf(''end'');');

	result = control_figure;
			
elseif strcmp(cmd, 'end')

	w = findobj('tag', 'grafsurf_range');
	for i = 1:length(w)
		delete(w(i))
	end

	w = findobj('tag', 'grafsurf_control');
	for i = 1:length(w)
		delete(w(i))
	end

end
