% GRAMM SCHMIDT
%
% Given three vectors this module performs the Gramm-Schmidt 
% orthogonalization process on them, then shows an animation.
% Enter the three vectors then click the 'Gramm Schmidt' 
% button.  To see the animation again, click 'Undo' and
% then 'Gramm Schmidt' again. 
%
% USES: arrow.m, an m-file available from the MATLAB web site. 

function GrammSchmidt (action)

persistent hU hV hW ...
   h_Ux h_Uy h_Uz h_Vx h_Vy h_Vz h_Wx h_Wy h_Wz h_axbox ...
	lUx lUy lUz lVx lVy lVz lWx lWy lWz

if ~nargin 
   action = 'Init' ;
end

switch (action)
case 'Init',
	fig = findobj ('Tag', 'GrammSchmidt') ;

	if length(fig) == 0
      r = CreateGrammSchmidtWindow ;
      h_Ux = r(1) ;
      h_Uy = r(2) ;
      h_Uz = r(3) ;
      h_Vx = r(4) ;
      h_Vy = r(5) ;
      h_Vz = r(6) ;
      h_Wx = r(7) ;
      h_Wy = r(8) ;
      h_Wz = r(9) ;
      h_axbox = r(10) ;
      lUx = str2num(cleanup(get(h_Ux, 'string'))) ;
      lUy = str2num(cleanup(get(h_Uy, 'string'))) ;
      lUz = str2num(cleanup(get(h_Uz, 'string'))) ;
      lVx = str2num(cleanup(get(h_Vx, 'string'))) ;
      lVy = str2num(cleanup(get(h_Vy, 'string'))) ;
      lVz = str2num(cleanup(get(h_Vz, 'string'))) ;
      lWx = str2num(cleanup(get(h_Wx, 'string'))) ;
      lWy = str2num(cleanup(get(h_Wy, 'string'))) ;
      lWz = str2num(cleanup(get(h_Wz, 'string'))) ;
      
      light ('Position', [0 -2 1]) ;
      GrammSchmidt ('Update') ;
      view ([30 10]) 
   else
      figure (fig)
   end
   
   
case 'GrammSchmidt',
   if isempty(lUx)
	   lUx = str2num(cleanup(get(h_Ux, 'string'))) ;
   	lUy = str2num(cleanup(get(h_Uy, 'string'))) ;
	   lUz = str2num(cleanup(get(h_Uz, 'string'))) ;
	   lVx = str2num(cleanup(get(h_Vx, 'string'))) ;
	   lVy = str2num(cleanup(get(h_Vy, 'string'))) ;
	   lVz = str2num(cleanup(get(h_Vz, 'string'))) ;
	   lWx = str2num(cleanup(get(h_Wx, 'string'))) ;
	   lWy = str2num(cleanup(get(h_Wy, 'string')));
      lWz = str2num(cleanup(get(h_Wz, 'string'))) ;
   end
   d=det([lUx lUy lUz; lVx lVy lVz; lWx lWy lWz]);
   if d==0 | d==NaN
      errordlg ('Please enter three linearly independent vectors.')
      return
   end
   
   
   % compute new V.
   % move to new position.
   Ux = str2num (cleanup(get(h_Ux,'string'))) ;
   Uy = str2num (cleanup(get(h_Uy,'string'))) ;
   Uz = str2num (cleanup(get(h_Uz,'string'))) ;
   Vx = str2num (cleanup(get(h_Vx,'string'))) ;
   Vy = str2num (cleanup(get(h_Vy,'string'))) ;
   Vz = str2num (cleanup(get(h_Vz,'string'))) ;
   
   a = dot ([Vx Vy Vz],[Ux Uy Uz]) / dot ([Ux Uy Uz], [Ux Uy Uz]) ;
   start = [Vx Vy Vz] ;
   Vx = Vx - a*Ux ;  Vy = Vy - a*Uy ;  Vz = Vz - a*Uz ;
   stop = [Vx Vy Vz] ;
   set(h_Vx,'String', num2str(Vx)) ;
	set(h_Vy,'String', num2str(Vy)) ;
   set(h_Vz,'String', num2str(Vz)) ;
   if abs(a) > .01
      AnimateVector (hV, start, stop) ;
   end
      
   
   % compute new W
   % move to new position.
   Wx = str2num(cleanup(get(h_Wx,'String'))) ;
   Wy = str2num(cleanup(get(h_Wy,'String'))) ;
   Wz = str2num(cleanup(get(h_Wz,'String'))) ;
   start = [Wx Wy Wz] ;
   b = dot ([Wx,Wy,Wz],[Vx,Vy,Vz]) / dot([Vx,Vy,Vz],[Vx,Vy,Vz]) ;
   c = dot ([Wx,Wy,Wz],[Ux,Uy,Uz]) / dot([Ux,Uy,Uz],[Ux,Uy,Uz]) ;
	Wx = Wx - b * Vx - c * Ux ;
   Wy = Wy - b * Vy - c * Uy ;
   Wz = Wz - b * Vz - c * Uz ;
   stop = [Wx Wy Wz] ;
   set(h_Wx,'String', num2str(Wx)) ;
	set(h_Wy,'String', num2str(Wy)) ;
   set(h_Wz,'String', num2str(Wz)) ;
   if abs(b) > .01 | abs(c) > .01
      AnimateVector (hW, start, stop) ;
   end 
%   GrammSchmidt ('Update') 
   
case 'Update',
   try
   lUx = '' ;
      
   Ux = str2num (cleanup(get(h_Ux,'string'))) ;
   Uy = str2num (cleanup(get(h_Uy,'string'))) ;
   Uz = str2num (cleanup(get(h_Uz,'string'))) ;
   Vx = str2num (cleanup(get(h_Vx,'string'))) ;
   Vy = str2num (cleanup(get(h_Vy,'string'))) ;
   Vz = str2num (cleanup(get(h_Vz,'string'))) ;
   Wx = str2num (cleanup(get(h_Wx,'string'))) ;
   Wy = str2num (cleanup(get(h_Wy,'string'))) ;
   Wz = str2num (cleanup(get(h_Wz,'string'))) ;
   
   d=det([Ux Uy Uz; Vx Vy Vz; Wx Wy Wz]);
   if d==0 | d==NaN
      errordlg ('Please enter three linearly independent vectors.')
      return
   end
   
	o = 2 ;   
   % uncomment the following line to make axis limits dynamic.  
	%  o = max([Ux Uy Uz Vx Vy Vz Wx Wy Wz]) ;
   axis(o*[-1 1 -1 1 -1 1]) 
   delete(get(gca,'children'))
	hU = PlotVector (Ux,Uy,Uz,'r') ;
	hV = PlotVector (Vx,Vy,Vz,'b') ;
	hW = PlotVector (Wx,Wy,Wz,'g') ;
   
   if get(h_axbox,'value') == 0
      axis off
   else
	   axis on
      xlabel ('X') 
	   ylabel ('Y') 
   	zlabel ('Z') 
	   grid on 
   end
   rotate3d on 
catch
   errordlg('Error in input.') 
   error(lasterr) 
end


case 'Help',
   helpwin ('GrammSchmidt.m') ;
   
case 'Undo', 
   if ~isempty(lUx)
	   set(h_Ux,'String', num2str(lUx)) ;
	   set(h_Uy,'String', num2str(lUy)) ;
   	set(h_Uz,'String', num2str(lUz)) ;
	   set(h_Vx,'String', num2str(lVx)) ;
	   set(h_Vy,'String', num2str(lVy)) ;
	   set(h_Vz,'String', num2str(lVz)) ;
	   set(h_Wx,'String', num2str(lWx)) ;
	   set(h_Wy,'String', num2str(lWy)) ;
	   set(h_Wz,'String', num2str(lWz)) ;
      GrammSchmidt('Update') ;
   end
   

case 'Reset',
   set(h_Ux,'String', '1') ;
   set(h_Uy,'String', '0') ;
   set(h_Uz,'String', '0') ;
   set(h_Vx,'String', '0') ;
   set(h_Vy,'String', '1') ;
   set(h_Vz,'String', '0') ;
   set(h_Wx,'String', '0') ;
   set(h_Wy,'String', '0') ;
   set(h_Wz,'String', '1') ;
   GrammSchmidt('Update') ;
   
   
   
case 'Quit',
   delete (findobj ('tag', 'GrammSchmidt')) ;
   w=findobj ('tag', 'math_main_menu') ;
   if length(w)
      figure(w(1));
   end

end

   
function h = PlotVector (x, y, z, c)
h = arrow ([0,0,0], [x,y,z], 'width', 3, 'facecolor', c) ;   

% Move vector having handle h from its current position 
% start=[a b c] to the new position stop=[d e f]. 
function AnimateVector (h, start, stop) 
anim_steps = 10 ;

axis(axis) 
set(h,'erasemode','xor')
for t=0:1/anim_steps:1
   arrow (h, 'stop', (1-t)*start + t*stop) 
   drawnow
end
set(h,'erasemode','normal')


function r = CreateGrammSchmidtWindow()
col = [.6 .8 1.0] ;
ucol = [1 .8 .8] ;
vcol = [.8 .8 1] ;
wcol = [.8 1 .8] ;

h0 =  figure('Color','w', ...
	'Name','GRAMM-SCHMIDT ORTHOGONALIZATION', ...
	'NumberTitle','off', ...
	'Position',[0 0 620 420], ...
	'Resize','off', ...
   'Tag','GrammSchmidt', ...
   'Menubar', 'none');

uicontrol (h0, 'position', [0 0 190 430], ...
   'backgroundcolor', col,...
   'style', 'frame') ;

axes('Units','pixels', ...
	'Position',[250 30 370 380]);

uicontrol('Parent',h0, ...
	'BackgroundColor', col, ...
	'Position', [5 310 20 20], ...
   'String','U=', ...
   'Style','text', ...
   'ForegroundColor','r');
h_Ux = uicontrol('Parent',h0, ...
	'BackgroundColor',ucol, ...
	'Callback','GrammSchmidt(''Update'')', ...
	'ListboxTop',0, ...
	'Position', [25 310 45 20], ...
	'String','1', ...
	'Style','edit');
h_Uy = uicontrol('Parent',h0, ...
	'BackgroundColor', ucol, ...
	'Callback','GrammSchmidt(''Update'')', ...
	'Position', [75 310 45 20], ...
	'String','0', ...
	'Style','edit');
h_Uz = uicontrol('Parent',h0, ...
	'BackgroundColor', ucol, ...
	'Callback','GrammSchmidt(''Update'')', ...
	'Position', [125 310 45 20], ...
	'String','0', ...
   'Style','edit');



uicontrol('Parent',h0, ...
	'BackgroundColor', col, ...
   'Position',[5 280 20 20], ...
	'String','V=', ...
   'ForegroundColor', 'b', ...
	'Style','text');
h_Vx = uicontrol('Parent',h0, ...
	'BackgroundColor', vcol, ...
	'Callback','GrammSchmidt(''Update'')', ...
	'Position',[25 280 45 20] , ...
	'String','0', ...
   'Style','edit');
h_Vy = uicontrol('Parent',h0, ...
	'BackgroundColor',vcol, ...
	'Callback','GrammSchmidt(''Update'')', ...
	'Position',[75 280 45 20 ], ...
	'String','1', ...
   'Style','edit');
h_Vz = uicontrol('Parent',h0, ...
	'BackgroundColor',vcol, ...
   'Callback','GrammSchmidt(''Update'')', ...
	'Position', [125 280 45 20], ...
	'String','0', ...
	'Style','edit') ;



uicontrol('Parent',h0, ...
	'BackgroundColor', col, ...
	'Position',[5 250 20 20], ...
   'String','W=', ...
   'ForegroundColor', [0 .3 0], ...
	'Style','text');
h_Wx = uicontrol('Parent',h0, ...
	'BackgroundColor',wcol, ...
	'Callback','GrammSchmidt(''Update'')', ...
	'Position',[25 250 45 20], ...
	'String','0', ...
   'Style','edit');
h_Wy = uicontrol('Parent',h0, ...
	'BackgroundColor', wcol, ...
	'Callback','GrammSchmidt(''Update'')', ...
	'Position',[75 250 45 20 ], ...
	'String','0', ...
   'Style','edit');
h_Wz = uicontrol('Parent',h0, ...
	'BackgroundColor', wcol, ...
   'Callback','GrammSchmidt(''Update'')', ...
	'Position',[125 250 45 20], ...
	'String','1', ...
   'Style','edit');


h_axbox = uicontrol ('Parent', h0, ...
   'Style', 'checkbox', ...
   'Backgroundcolor', col, ...
   'value', 1, ...
   'String', 'Show Axis', ...
   'Position', [40 200 70 20], ...
   'Callback', 'GrammSchmidt(''Update'')') ;


uicontrol('Parent',h0, ...
	'Callback','GrammSchmidt(''GrammSchmidt'')', ...
	'Position',[40 150 108 30], ...
   'String','Gramm Schmidt');
uicontrol('Parent',h0, ...
	'Callback','GrammSchmidt(''Undo'')', ...
	'Position',[40 110 108 30], ...
   'String','Undo');
uicontrol('Parent',h0, ...
	'Callback','GrammSchmidt(''Help'')', ...
	'Position',[40 70 108 30], ...
	'String','Help');
uicontrol('Parent',h0, ...
	'Callback','GrammSchmidt(''Quit'')', ...
	'Position',[40 30 108 30], ...
	'String','Quit');



r = [h_Ux h_Uy h_Uz h_Vx h_Vy h_Vz h_Wx h_Wy h_Wz h_axbox] ;

