function r=impdef(cmd)
% IMPLICITLY DEFINED FUNCTIONS
%
% A module for graphing curves defined by f(x,y) = K.
%
% First, set the domain of x and y values.  Then, enter f(x,y) into the
% appropriate box.  Now, to draw level curves, you can click on the graoh
% to see the level curve through a given point.  Too draw a particular
% curve f(x,y) = K, type a number in the K box and press the "Use K=" button.
%
% 2.0/21-July-1998

%
% HISTORY: 1.0 -- Original by Julie Mitchell
%          2.0 -- new version, adapted from lagrange.m by Tom Insel 
%                 (summer 1998)
%
% REQUIRES: errmesg.m, cleanup.m
%
% TESTED: MATLAB 5.2 for Macintosh

persistent f_of_xy xlow xhi ylow yhi fig_axes level_text K_field

% colors

math_bkgnd = [0.6 0.8 1.0];
math_field = [1 1 1];
math_oldcurve = [1 0 0];

if ~nargin,

	main_figures = findobj('tag', 'impdef_main');
	
	if length(main_figures) == 0
		cmd = 'new';
	else
		figure(main_figures(1));
		cmd = 'NOCOMMAND';
	end

end

if strcmp(cmd,'new')

	main_figures = findobj('tag', 'impdef_main');
	if length(main_figures) == 0
		main_figures = impdef('makecontrol');
	end

	main_figure = main_figures(1);	
	
	impdef('draw');
	
elseif strcmp(cmd,'makecontrol')

	fig_han = figure(...
		'name', 'Implicitly Defined Functions',...
		'Tag', 'impdef_main',...
		'numbertitle', 'off',...
		'position', [10 10 600 400],...
		'color', math_bkgnd,...
		'resize', 'off',...
		'renderer', 'zbuffer',...
		'WindowButtonDownFcn', 'impdef(''click'')');

	uicontrol(fig_han,...
		'style', 'text',...
		'string', 'FUNCTION',...
		'horizontalalignment', 'left',...
		'backgroundcolor', math_bkgnd,...
		'position', [10 370 180 20]);

	uicontrol(fig_han,...
		'style', 'text',...
		'horizontalalignment', 'right',...
		'string', 'f(x,y) = K = ',...
		'backgroundcolor', math_bkgnd,...
		'position', [10 350 70 20]);

	f_of_xy = uicontrol(fig_han,...
		'style', 'edit',...
		'backgroundcolor', math_field,...
		'string', 'x^2 - 2*y^2',...
		'position', [80 350 110 20],...
		'callback', 'impdef(''draw'')');
		
	level_text = uicontrol(fig_han,...
		'style', 'text',...
		'backgroundcolor', math_bkgnd,...
		'position', [10 320 180 20]);
		
	uicontrol(fig_han,...
		'string', 'Use K = ',...
		'callback', 'impdef(''level'')',...
		'position', [10 285 50 25]);
		
	K_field = uicontrol(fig_han,...
		'style', 'edit',...
		'backgroundcolor', math_field,...
		'string', '-0.25',...
		'position', [70 285 80 20]);

	uicontrol(fig_han,...
		'style', 'frame',...
		'backgroundcolor', 'k',...
		'position', [10 275 180 1]);

	uicontrol(fig_han,...
		'style', 'text',...
		'string', 'DOMAIN',...
		'horizontalalignment', 'left',...
		'backgroundcolor', math_bkgnd,...
		'position', [10 245 180 20]);

	xlow = uicontrol(fig_han,...
		'style', 'edit',...
		'backgroundcolor', math_field,...
		'string', '-1',...
		'position', [20 220 45 20],...
		'callback', 'impdef(''draw'')');

	if strcmp(computer,'MAC2')
		domain_string = ' x ';
	else
		domain_string = '<= x <=';
	end
		
	uicontrol(fig_han,...
		'style', 'text',...
		'string', domain_string,...
		'backgroundcolor', math_bkgnd,...
		'position', [65 220 50 20]);
		
	xhi = uicontrol(fig_han,...
		'style', 'edit',...
		'backgroundcolor', math_field,...
		'string', '1',...
		'position', [115 220 45 20],...
		'callback', 'impdef(''draw'')');	

	ylow = uicontrol(fig_han,...
		'style', 'edit',...
		'backgroundcolor', math_field,...
		'string', '-1',...
		'position', [20 195 45 20],...
		'callback', 'impdef(''draw'')');	

	if strcmp(computer,'MAC2')
		domain_string = ' y ';
	else
		domain_string = '<= y <=';
	end
		
	uicontrol(fig_han,...
		'style', 'text',...
		'string', domain_string,...
		'backgroundcolor', math_bkgnd,...
		'position', [65 195 50 20]);
		
	yhi = uicontrol(fig_han,...
		'style', 'edit',...
		'backgroundcolor', math_field,...
		'string', '1',...
		'position', [115 195 45 20],...
		'callback', 'impdef(''draw'')');

	uicontrol(fig_han,...
		'style', 'frame',...
		'backgroundcolor', 'k',...
		'position', [10 145 180 1]);

	uicontrol(fig_han,...
		'string', 'Sketch Level Curves',...
		'position', [10 100 180 35],...
		'callback', 'impdef(''sketch'');');

	uicontrol(fig_han,...
		'string', 'Clear Level Curves',...
		'position', [10 55 180 35],...
		'callback', 'impdef(''draw'');');

	uicontrol(fig_han,...
		'string', 'Help',...
		'position', [10 10 85 35],...
		'callback', 'helpwin impdef');

	uicontrol(fig_han,...
		'string', 'Quit',...
		'position', [105 10 85 35],...
		'callback', 'impdef(''end'');');

	fig_axes = axes(...
		'Position',[.41 .075 .56 .86],...
		'XLimMode', 'auto',...
		'YLimMode', 'auto');
		
	r = fig_han;

elseif strcmp(cmd, 'draw') | strcmp(cmd, 'click') | strcmp(cmd, 'level') | strcmp(cmd, 'sketch')

	main_figures = findobj('tag', 'impdef_main');
	if length(main_figures) == 0
		main_figures = impdef('makecontrol');
	end

	main_figure = main_figures(1);

	err_field=0;
	err_type=0;

	x = -666; y = -666; % test values for x,y (danger: division by zero?)
	
	xlow_string = cleanup(get(xlow,'string'));
	xhi_string = cleanup(get(xhi,'string'));
	ylow_string = cleanup(get(ylow,'string'));
	yhi_string = cleanup(get(yhi,'string'));
	f_string = cleanup(get(f_of_xy,'string'));

	eval([xlow_string ';'], 'err_field=''low x''; err_type=''num'';')
	eval([xhi_string ';'], 'err_field=''high x''; err_type=''num'';')
	eval([ylow_string ';'], 'err_field=''low y''; err_type=''num'';')
	eval([yhi_string ';'], 'err_field=''high y''; err_type=''num'';')
	eval([f_string ';'], 'err_field=''f''; err_type=''fun'';')

	if err_field
		errmesg(err_field, err_type);
		error(lasterr);
		return
	end

	xlow_n = eval(xlow_string);
	xhi_n = eval(xhi_string);
	ylow_n = eval(ylow_string);
	yhi_n = eval(yhi_string);
	
	% hi/low make sure in the right order

	if strcmp(cmd, 'draw')

		title('Click to draw a level curve for f.');
		set(level_text, 'string', []);
		cla
		axis([xlow_n xhi_n ylow_n yhi_n])
			
	elseif strcmp(cmd, 'click')
		
		p = get(fig_axes, 'CurrentPoint');
		click_x = p(1, 1);
		click_y = p(1, 2);
		
		x = click_x;
		y = click_y;
		f1 = eval(f_string);
		
		[x, y] = meshgrid(linspace(xlow_n,xhi_n), linspace(ylow_n,yhi_n));
		f = eval(vectorize(f_string));
		
		hold on
		
		set(findobj(gca, 'tag', 'levelset'), 'color', math_oldcurve, 'tag', 'oldlevelset')
		
		[c,h] = contour(x, y, f, [f1 f1], 'b');
		set(h, 'tag', 'levelset')
		
		hold off

		set(level_text, 'string',...
			['f(' num2str(click_x,2) ',' num2str(click_y,2) ') = ' num2str(f1,5)]);
			
	elseif strcmp(cmd, 'level')

		try
			f1 = str2num(cleanup(get(K_field, 'string')));
		catch
			errmesg('K', 'num');
			return
		end

		[x, y] = meshgrid(linspace(xlow_n,xhi_n), linspace(ylow_n,yhi_n));
		f = eval(vectorize(f_string));
		
		hold on

		set(findobj(gca, 'tag', 'levelset'), 'color', math_oldcurve, 'tag', 'oldlevelset')
		
		[c,h] = contour(x, y, f, [f1 f1], 'b');
		set(h, 'tag', 'levelset')
		
		hold off

		set(level_text, 'string', ['f = ' num2str(f1,5)]);
		
		elseif strcmp(cmd, 'sketch')
	
			[x, y] = meshgrid(linspace(xlow_n,xhi_n), linspace(ylow_n,yhi_n));
			f = eval(vectorize(f_string));
			
			hold on
	
			set(findobj(gca, 'tag', 'levelset'), 'color', math_oldcurve, 'tag', 'oldlevelset')
			
			[c,h] = contour(x, y, f, 5, 'k:');
			set(h, 'color', [.7 .7 .7]);
			
			hold off
	
			set(level_text, 'string', []);
	
		end
		
elseif strcmp(cmd, 'end')

	w = findobj('tag', 'impdef_main');
	for i = 1:length(w)
		delete(w(i))
	end

	% if it's open, return to the main menu

	w = findobj('tag', 'math_main_menu');
	if length(w)
		figure(w(1));
	end

end;
