function result=inflectious(cmd,x)
% INFLECTIOUS
%
% A module for graphing a function and it's derivatives.
%
% Enter the function you wish to graph in the f(x) boxes. Check the
% boxes for each of f(x), f'(x), and f''(x) that you wish to see
% graphed.
%
% Click on the graph to zoom in, shift-click to zoom out.
%
% Colors: f(x) is plotted in blue, f'(x) in green, and f''(x) in red.
%
% 2.0/1-Aug-1998

% HISTORY:
% 1.0 -- original by Julie Mitchell (11-29-26)
% 2.0 -- rewritten by tinsel (august 1998) based on my version 2.0 of
%        Julie's EZPlot module.

math_bkgnd = [0.6 0.8 1.0];
math_field = [1 1 1];
math_axis = [0.6 0.6 0.6];

persistent x0_text x1_text f_text g_field h_field f_check g_check h_check grid_check our_axes

if ~nargin,
	
	control_figures = findobj('tag', 'inflectious_main');
	
	if length(control_figures) == 0
		cmd = 'new';
	else
		figure(control_figures(1));
		cmd = 'NOCOMMAND';
	end

end

if strcmp(cmd, 'new')
	
	fig = figure(...
		'name', 'Inflectious',...
		'Tag', 'inflectious_main',...
		'numbertitle', 'off',...
		'renderer', 'zbuffer',...
		'position', [10 10 600 410],...
		'color', math_bkgnd,...
		'resize', 'off');
	
	uicontrol(fig,...
		'style','frame',...
		'position', [10 90 580 1]);
	
	f_check = uicontrol(fig,...
		'style', 'checkbox',...
		'string', 'f(x) =',...
		'backgroundcolor', math_bkgnd,...
		'value', 1,...
		'callback', 'inflectious(''redraw'')',...
		'position', [40 60 50 20]);
	
	f_text = uicontrol(fig,...
		'style', 'edit',...
		'string', 'sin(x)',...
		'backgroundcolor', math_field,...
		'position', [100 60 290 20],...
		'horizontalalignment', 'left',...
		'callback', 'inflectious(''redraw'')');
	
	g_check = uicontrol(fig,...
		'style', 'checkbox',...
		'string', 'g(x) =',...
		'backgroundcolor', math_bkgnd,...
		'callback', 'inflectious(''redraw'')',...
		'position', [40 35 50 20]);
	
	g_field = uicontrol(fig,...
		'style', 'text',...
		'string', '',...
		'backgroundcolor', math_bkgnd,...
		'position', [100 35 290 20],...
		'horizontalalignment', 'left',...
		'callback', 'inflectious(''redraw'')');
	
	h_check = uicontrol(fig,...
		'style', 'checkbox',...
		'string', 'h(x) =',...
		'backgroundcolor', math_bkgnd,...
		'callback', 'inflectious(''redraw'')',...
		'position', [40 10 50 20]);
	
	h_field = uicontrol(fig,...
		'style', 'text',...
		'string', '',...
		'backgroundcolor', math_bkgnd,...
		'position', [100 10 290 20],...
		'horizontalalignment', 'left',...
		'callback', 'inflectious(''redraw'')');
	
	x0_text = uicontrol(fig,...
		'style', 'edit',...
		'string', '-pi',...
		'backgroundcolor', math_field,...
		'position', [410 60 50 20],...
		'callback', 'inflectious(''redraw'')');
	
	if strcmp(computer,'MAC2')
		domain_string = ' x ';
	else
		domain_string = '<= x <=';
	end
		
	uicontrol(fig,...
		'style', 'text',...
		'string', domain_string,...
		'horizontalalignment', 'center',...
		'backgroundcolor', math_bkgnd,...
		'position', [460 60 40 20]);
	
	x1_text = uicontrol(fig,...
		'style', 'edit',...
		'string', 'pi',...
		'backgroundcolor', math_field,...
		'position', [500 60 50 20],...
		'callback', 'inflectious(''redraw'')');
	
	grid_check = uicontrol(fig,...
		'style', 'checkbox',...
		'string', 'Draw Grid',...
		'backgroundcolor', math_bkgnd,...
		'callback', 'inflectious(''redraw'')',...
		'position', [410 40 150 20]);
	
	uicontrol(fig,...
		'string', 'Help',...
		'position', [410 10 85 25],...
		'callback', 'helpwin inflectious');
	
	uicontrol(fig,...
		'string', 'Quit',...
		'position', [500 10 85 25],...
		'callback', 'inflectious(''end'')');

	our_axes = 0;
	
	inflectious('redraw');
	
	if ~havesymbolic
		warndlg({'The Symbolic Toolkit is not installed, so the module',...
			'INFLECTIOUS will not be very useful.'},...
			'Warning');
		set([g_check h_check], 'value', 0, 'enable', 'off');
	end
		
	if nargout, result = fig, end
	
elseif strcmp(cmd,'redraw')

	control_figures = findobj('tag', 'inflectious_main');
	if length(control_figures) == 0
		control_figures = inflectious('new');
	end

	fig = control_figures(1);
	
	try
		x0 = str2num(cleanup(get(x0_text, 'string')));
		x1 = str2num(cleanup(get(x1_text, 'string')));
	catch
		errmesg('x0 or x1', 'num')
		error(lasterr)
	end

	% recreate axes here to get rid of the same zooming bug that seems
	% to affect other modules.
	if our_axes, delete(our_axes), end
	our_axes = axes('position', [.05 .3 .9 .65]);

	hold on
	
	if get(f_check, 'value')
		try
			f_string = cleanup(get(f_text, 'string'));
			ezplot(f_string, [x0 x1], fig);
		catch
			errmesg('f(x)', 'fun')
			error(lasterr)
		end	
	
		set(findobj(fig, 'type', 'line','tag',''), 'tag', 'colored')	
	end

	if havesymbolic 
		g_string = diff(f_string, 'x');
		set(g_field, 'string', char(g_string));
		h_string = diff(g_string, 'x');
		set(h_field, 'string', char(h_string));
	else
		set([g_field h_field], 'string', 'NEED SYMBOLIC TOOLKIT');
	end
	
	if get(g_check, 'value')
		ezplot(g_string, [x0 x1], fig);
		set(findobj(fig, 'type', 'line','tag',''), 'color', 'g', 'tag', 'colored')	
	end	

	if get(h_check, 'value')
		ezplot(h_string, [x0 x1], fig);
		set(findobj(fig, 'type', 'line','tag',''), 'color', 'r', 'tag', 'colored')	
	end

	if get(grid_check, 'value')
		grid on
	else
		grid off
		plot([x0 x1], [0 0], 'k:')
	end
	
	hold off
	zoom on
	title(''), ylabel(''), xlabel('')
	set(gca, 'YLimMode', 'auto')
	
elseif strcmp(cmd, 'end')

	w = findobj('tag', 'inflectious_main');
	for i = 1:length(w)
		delete(w(i))
	end

	% if it's open, return to the main menu
	
	w = findobj('tag', 'math_main_menu');
	if length(w)
		figure(w(1));
	end

end		

function r = havesymbolic()
r = ~isempty(findstr(path,'symbolic'));
