% ODESYS2D
% This module plots solutions to a 2x2 system x' = Ax of linear ODE's.
%
% First type in A then enter the range of time values over which to plot
% solution curves, and finally click the Update button.  If the graph
% appears blocky try increasing the resolution using the slider.  If the
% axis are misleadingly skewed click the checkbox to force the axis to be
% square.  
%
% The module computes two fundamental solutions, x1 and x2 and then plots 
% linear combinations a x1 + b x2 such that a^2 + b^2 = 1.
   
% REQUIRES: cleanup.m, errmesg.m
% EXAMPLE: Type odesys2d to run this module.
% AUTHOR: (?), 1996. 
%		  Revised/ported: William Stein, 1998.
% HISTORY: rewritten February, 1998; 
%		  	  updated, bug fixes, June 1998.

function odesys2d(cmd,A,rtime,t0)

persistent h_a h_b h_c h_d h_t0 h_t1 h_ax h_fig h_axisequal h_res

math_bkgnd = [0.6 0.8 1.0] ;

if ~nargin
   cmd	=	'new';
   A	=	[.5 1 ; 0 -.5];
   rtime	=	3;
end;

switch cmd
case 'new',
   figs=findobj('tag','tag_odesys2d');
   if length(figs) ~= 0
      figure(figs(1));
      return;
   end
   
   h_fig=figure('Position',[20 40 763 514], ... %[10 10 600 400],...
      'Tag','tag_odesys2d',...
      'Color', math_bkgnd, ...
      'NumberTitle','off','Resize','off',...
      'Menu', 'none',...
      'Name','2D SYSTEM OF ODEs');
   
   % Create the objects: axes, Matrix input boxes,
   % ending time value, [Draw Solutions] button,
   % and [Help] button.
   
   % Create axes
   h_ax=axes('Units','pixels','Position',[270 60 450 400]);   
   
   % Create control frames
   % x'=Ax
   uicontrol(h_fig,'Style','text',...
      'Position', [20 470 60 15],...
      'HorizontalAlignment', 'Left',...
      'String', 'x'' = Ax',...
      'Background', math_bkgnd);
   
   % A=[  ]  [  ]
   %   [  ]  [  ]
   uicontrol(h_fig,'Style','text',...
      'Position',[20 415 60 15],...
      'String','A=',...
      'HorizontalAlignment', 'Left',...
      'Background', math_bkgnd);
   
   uicontrol(h_fig,'Style','frame',...
      'Position',[40 394 1 51]);
   uicontrol(h_fig,'Style','frame',...
      'Position',[40 394 6 1]);
   uicontrol(h_fig,'Style','frame',...
      'Position',[40 445 6 1]);
   uicontrol(h_fig,'Style','frame',...
      'Position',[144 394 1 51]);
   uicontrol(h_fig,'Style','frame',...
      'Position',[139 394 6 1]);
   uicontrol(h_fig,'Style','frame',...
      'Position',[139 445 6 1]);
   
   h_a=uicontrol(h_fig,'Style','edit',...
      'Position',[50 423 37 20],...
      'Background','w',...
      'String',num2str(A(1,1)));
   h_b=uicontrol(h_fig,'Style','edit',...
      'Position',[97 423 37 20],...
      'Background','w',...
      'String',num2str(A(1,2)));
   h_c=uicontrol(h_fig,'Style','edit',...
      'Position',[50 397 37 20],...
      'Background','w',...
      'String',num2str(A(2,1)));
   h_d=uicontrol(h_fig,'Style','edit',...
      'Position',[97 397 37 20],...
      'Background','w',...
      'String',num2str(A(2,2)));
   
   % "Plot from t = 0 to [ ]"
   
   uicontrol(h_fig,'Style','text',...
      'String','Range of t values',...
      'HorizontalAlignment', 'Left',...
      'Position',[20 360 140 15],...
      'Background',math_bkgnd);
   
   h_t0=uicontrol(h_fig,'Style','edit',...
      'Position',[50 335 50 20],...
      'Background','w',...
      'HorizontalAlignment', 'Left',...
      'String','0');
   
   h_t1=uicontrol(h_fig,'Style','edit',...
      'Position',[108 335 50 20],...
      'Background','w',...
      'HorizontalAlignment', 'Left',...
      'String',num2str(rtime));
   
   uicontrol (h_fig, 'style', 'text',...
      'position', [20 305 80 16], ...
      'string', 'Resolution', ....
      'horizontalalignment', 'left',...
      'background', math_bkgnd) ;
   
   h_res=uicontrol(h_fig,'Style','slider',...
      'Position',[20 285 140 16],...
      'min', 20, 'max', 450, 'value', 25) ;
   
   h_axisequal = uicontrol (h_fig, 'style', 'checkbox',...
      'position', [20 250 130 16],...
      'value', 0, ...
      'background', math_bkgnd, ...
      'string', 'Force square axis') ;
   
   % [Draw Solutions]
   uicontrol(h_fig,'String','Update',...
      'Position',[40 170 120 40],...
      'Callback','odesys2d(''DrawSolutions'')');
   
   %uicontrol(h_fig,'String','Library',...
   %   'Position',[40 65 120 30],...
   %   'Callback','odesys2d(''Library'')');
   
   % [Help]
   uicontrol(h_fig,'String','Help',...
      'Position',[40 100 120 30],...
      'Callback','odesys2d(''Help'')');
   
   uicontrol(h_fig,'String','Quit',...
      'Position',[40 30 120 30],...
      'Callback','delete(gcbf)');
   
   odesys2d('plot',A,0,rtime);
   
case 'DrawSolutions',
   
   s_a=cleanup(get(h_a,'String'));
   s_b=cleanup(get(h_b,'String'));
   s_c=cleanup(get(h_c,'String'));
   s_d=cleanup(get(h_d,'String'));
   s_t0=cleanup(get(h_t0,'string'));
   s_t=cleanup(get(h_t1,'string'));
   
   field=0;
   eval([s_a ';'], 'field=''Upper left entry of A'';');
   eval([s_b ';'], 'field=''Upper right entry of A'';');
   eval([s_c ';'], 'field=''Lower left entry of A'';');
   eval([s_d ';'], 'field=''Lower right entry of A'';');
   eval([s_t0 ';'], 'field=''Starting time'';');
   eval([s_t ';'], 'field=''Ending time'';');
   
   if field
      errmesg(field,'num'); error(lasterr);
   end      
   
   a=str2num(s_a); b=str2num(s_b); 
   c=str2num(s_c); d=str2num(s_d);
   t0=str2num(s_t0); t=str2num(s_t) ;
   
   odesys2d('plot',[a b;c d],t,t0); 
   
case 'plot',
   set (gcbf, 'pointer', 'watch') ;
   A_size	=	size(A);
   A_rownum	=	A_size(1);
   
   [V,D]	=	eig(A);	%%% vectors V and values D(i,i)
   
   if A_rownum	==	2	
      
      eigval1	= D(1,1);
      eigval2	= D(2,2);
      
      eigvec1	=	V(:,1);
      eigvec2	=	V(:,2);
      
      t	=	linspace(t0,rtime,get (h_res, 'value'));
      
      t	=	t';				%%% transpose
      
      if (imag(eigval1)==0)&~(eigval1==eigval2) %%% exponential
         
         f1	=	exp(eigval1.*t);
         f2	=	exp(eigval2.*t);
         
         soln1	=	[f1.*eigvec1(1) f1.*eigvec1(2)];
         soln2	=	[f2.*eigvec2(1) f2.*eigvec2(2)];
         
      elseif eigval1==eigval2		%%% repeated root  note:this needs 3 cases
         
         if rank(V)==2			%%% two eigenvectors
            f1	=	exp(eigval1.*t);
            f2	=	f1;
            soln1	=	[f1.*eigvec1(1) f1.*eigvec1(2)];
            soln2	=	[f2.*eigvec2(1) f2.*eigvec2(2)];
         elseif rank(V)==1		%%% one eigenvector
            f1	=	exp(eigval1.*t);
            f2	=	t.*f1;
            
            lambdaI	=	eigval1.*diag([1 1]);
            
            orthvec	=	[eigvec1(2) -eigvec1(1)];
            compvec	=	(A-lambdaI)*orthvec';
            
            if compvec(1)==0
               eigvec2	=	orthvec.*(eigvec1(2)/compvec(2));
            else
               eigvec2	=	orthvec.*(eigvec1(1)/compvec(1));
            end;
            
            
            soln1	=	[f1.*eigvec1(1) f1.*eigvec1(2)];
            soln2	=	[f2.*eigvec1(1) f2.*eigvec1(2)] + [f1.*eigvec2(1) f1.*eigvec2(2)];
            
            soln2	=	soln2 - (orthvec(1)-soln2(1)).*soln1; %%% canonical one
            
         elseif rank(V)==0	%%% double zeros
            f1	=	0.*t + 1;
            f2	=	0.*t;
            soln1	=	[f1 f2];
            soln2	=	[f2 f1];
         end;
         
      else						%%% complex roots
         f1	=	exp(eigval1.*t);
         
         tempvec	=	[f1.*eigvec1(1) f1.*eigvec1(2)];
         
         soln1	=	real(tempvec);
         soln2	=	imag(tempvec);
         
      end;
      
      plotvals1	=	[soln1(:,1) soln2(:,1)];
      plotvals2	=	[soln1(:,2) soln2(:,2)];
      
      s	=	linspace(0,2*pi,40);
      
      coeffs	=	[cos(s) sin(s)];
      
      plotvals3	=	[];
      plotvals4	=	[];
      
      for	i	=	1:length(s)
         newsoln	=	cos(s(i)).*soln1 + sin(s(i)).*soln2;
         plotvals3	=	[plotvals3 newsoln(:,1)];
         plotvals4	=	[plotvals4 newsoln(:,2)];
      end;
      
      
      
      plot (plotvals3, plotvals4) ;
      
      title('Solutions to x'' = A x','color','b','FontSize',14);
      xlabel('Click and drag to zoom in.');
      
      hold on
      plot(plotvals3,plotvals4,'b');
      plot(plotvals1(:,1),plotvals2(:,1),'r');
      plot(plotvals1(:,2),plotvals2(:,2),'r');
      datalen	=	length(plotvals1(:,1));
      plot(plotvals1(datalen,1),plotvals2(datalen,1),'r');%'wo');
      plot(plotvals1(datalen,2),plotvals2(datalen,2),'r');%'wo');
      
      if get(h_axisequal, 'value') == 1
         axis equal 
      end
      
      grid on
      hold off, drawnow
      zoom on	
      set (gcbf, 'pointer', 'arrow') ;
      
   else
      
   end;
   
case 'Help',
   helpwin ('odesys2d') 
   
case 'Library',
   msgbox ('unimplemented') ;
end;



