% POSITION
%
%    In an upcoming mission, Agent 17, you will be in a train riding through 
% the mountains in a cabin which has no outside windows.   It is extremely 
% important that you know where you are as you ride along.   Fortunately
% the car is equipped with a speedometer so you know how fast the
% train is traveling at any given time.  This is enough information 
% for you to determine how far you have traveled since the trip began. 
% This module will help you to practice this skill (integration).   
%
% INSTRUCTIONS:
%    1. In the first axis sketch a curve which represents your velocity
%       as a function of time.   Start by clicking the mouse button
%       once on the left side of the graph.  Release the mouse button
%       and move the mouse to the right drawing the curve.  If the
%       curve doesn't look right you can move the mouse backwards
%       to undo what you have just drawn.  To finish drawing the 
%       curve move the mouse off the axis to the right and wait a moment. 
%    2. Next try to sketch the distance traveled as a function of time. 
%    3. When you are done the computer will sketch the actual distance.
%       Compare the two and keep practicing until you can make them very
%       close for many different velocity graphs. 
%
% William Stein (was@math.berkeley.edu), Summer 1998

function position (action)

res = 600 ;
xmin = 0 ;
xmax = 5 ;
ymin = 0 ;
ymax = 60 ;

persistent h hline stage

if nargin == 0
   action = 'init' ;
end

switch action
   
case 'init',
   fig = findobj ('Tag', 'Position') ;
   if isempty(fig)
      h = CreatePositionWindow ;
      h.userline1 = 0 ;
      h.userline2 = 0 ;
      h.answer = 0 ;
      
      % the stages are ...
      % 1: the user draws a function in the top axes
      % 2: then the integral in the bottom axes
      % 3: finally, the user can click on a button to 
      %    compute the answer. 
      % 4: the answer is displayed.
      position 'reset'
      
   else
      h.Fig = fig(1);
      figure (h.Fig) 
   end   
   
case 'start'
   if stage == 1 
   	loc = get(gcf, 'CurrentPoint') ;
   	r = get(h.ax1,'Position') ;
      
      if (loc(1) > r(1)) & (loc(1) < r(1)+r(3)) & ...
            (loc(2) > r(2)) & (loc(2) < r(2)+r(4))       
	      set (gcbf, 'pointer', 'crosshair') 
         set (gcbf, 'WindowButtonMotionFcn', 'position(''move'')')
         set (gcbf, 'WindowButtonDownFcn', '')
         
         axis manual
         hold on
         
         if h.userline1
            set (h.userline1, 'Erasemode', 'Normal') 
            delete(h.userline1)
         end
         
         pnt = get(gca,'CurrentPoint');
         h.userline1 = plot (pnt(1), pnt(3), 'Erasemode', 'None');
         position 'move'
      end
      
   elseif stage == 2 
   	loc = get(gcf, 'CurrentPoint') ;
   	r = get(h.ax2,'Position') ;
      if (loc(1) > r(1)) & (loc(1) < r(1)+r(3)) & ...
            (loc(2) > r(2)) & (loc(2) < r(2)+r(4))       
   	   set (gcbf, 'pointer', 'crosshair') 
	      set (gcbf, 'WindowButtonMotionFcn', 'position move')
         set (gcbf, 'WindowButtonDownFcn', '')
         axis manual
         hold on
         if h.userline2
            set (h.userline2, 'Erasemode', 'Normal') 
            delete(h.userline2)
         end
         pnt = get(gca,'CurrentPoint');
         h.userline2 = plot (pnt(1), pnt(3), 'Erasemode', 'None', 'color','red');
         position 'move'
      end
   end
   
case 'move'
   pnt = get(gca, 'CurrentPoint') ;
   if stage == 1
      hline = h.userline1 ;
   else
      hline = h.userline2 ;
   end
   
   xlim = get(gca, 'xlim') ;
   ylim = get(gca, 'ylim') ;
   if pnt(1) < xlim(1) | pnt(3) < ylim(1) | pnt(3)>ylim(2)
      return
   end

   x = get (hline, 'XData') ;
   y = get (hline, 'YData') ;
   if pnt(1) > x(length(x)) 
      set (hline, 'XData', [x pnt(1)], ...
         'YData', [y pnt(3)], 'erasemode', 'none') ;
      drawnow
   elseif pnt(1)>=0  % undo -- delete everything afterwards
      a = (x < pnt(1)) ;
      col = get(hline, 'color') ;
      set (hline, 'color', 'w') ;
      drawnow
	   x = x(a) ;
      y = y(a) ;
      if isempty(x)
         x = pnt(1) ;
         y = pnt(3) ;
      end
         
	   set (hline, 'color', col, 'xdata', x, 'ydata', y) 
      drawnow 
   end
   
   pnt = get(gca, 'CurrentPoint') ;
   if pnt(1) > xmax
      Position('stop') 
   end
      
case 'stop'
   if isempty(get(gcbf,'windowbuttonmotionfcn'))
      return
   end
   set (gcbf, 'pointer', 'arrow') ;
   set (gcbf, 'WindowButtonDownFcn', 'Position start')
   set (gcbf,'WindowButtonMotionFcn','')
   if stage == 1
      hline = h.userline1 ;
      stage = 2 ;
      position ('update_mode_dep') ;      
      position ('compute_answer') ;
   else
      hline = h.userline2 ;
      stage = 3 ;
      position ('update_mode_dep') ;
   end
      
   
case 'undo', 
   delete(h.userline2) 
   h.userline2 = 0 ;
   delete(h.answer) 
   h.answer = 0 ;
   stage = 1 ;
   
   set (gcbf, 'WindowButtonMotionFcn', 'position move')
   Position('stop') 
   
case 'reset',
   stage = 1 ;
   position ('update_mode_dep') 
   
case 'update_mode_dep',
   switch stage
   case 1,
      if h.userline1
         set (h.userline1, 'Erasemode', 'Normal') ;
         delete(h.userline1)
         h.userline1=0;
      end
      if h.userline2
         set (h.userline2, 'Erasemode', 'Normal') ;
         delete(h.userline2)
         h.userline2=0;
      end
      if h.answer
         delete(h.answer)
         h.answer=0;
      end
      set (h.ax1, 'XLim', [xmin xmax], 'YLim', [ymin ymax]) ;
      set (h.ax2, 'XLim', [xmin xmax], 'YLim', [ymin ymax]) ;
      set (h.DrawVelocityText,'visible','on') ;
      set ([h.undo h.ax2 h.DrawPositionText h.answerbutton], 'visible', 'off') ;
   case 2,
      set ([h.undo h.DrawVelocityText h.answerbutton], 'visible', 'off') ;
      set ([h.ax2 h.DrawPositionText], 'visible', 'on') ;
   case 3,
		set ([h.DrawVelocityText h.DrawPositionText], 'visible', 'off') ;
      set ([h.answerbutton h.undo], 'visible', 'on') ;
   case 4, 
      set ([h.undo h.DrawVelocityText h.DrawPositionText h.answerbutton], ...
         'visible', 'off') ;
   end      
   
	axes(h.ax1) 
   grid on
	xlabel('time (hours)') 
	ylabel('velocity (mph)')
   
   axes (h.ax2)
   grid on

   xlabel('time (hours)') 
	ylabel('miles from start point')

   drawnow
   
case 'display_answer',   
   stage = 4 ;
   set (h.answer, 'color',[0 .5 0]) ;
   drawnow 
   position ('update_mode_dep') 

case 'compute_answer'
   x = get (h.userline1, 'XData') ;
   y = get (h.userline1, 'YData') ;
   axes (h.ax2) ;
   hold off
   yy = zeros (1, length(x)) ;
   for i = 2:length(x)
      yy(i) = yy(i-1) + (x(i)-x(i-1))*y(i) ;
   end
   h.answer = plot (x, yy, 'color', 'r', 'color', 'w') ;
   set (h.ax2, 'XLim', [0,5]) ;
	position ('update_mode_dep') 
   
case 'help',
   helpwin position
   
case 'quit',
   delete (findobj ('tag', 'Position')) 
   w = findobj ('tag', 'math_main_menu') ;
   if w
      figure(w(1))
   end
end

      

function h=CreatePositionWindow ()
	
h.Fig = figure('position',[20 20 620 410],...
   'color', [.6 .8 1], ...
   'name', 'POSITION FROM VELOCITY', ...
   'numbertitle', 'off', ...
   'resize', 'off', ...
   'menubar', 'none', ...
   'WindowButtonDownFcn','Position(''start'')', ...
   'Tag', 'Position');

h.ax1 = axes ('parent', h.Fig, ...
   'Units','pixels', ...
   'Position',[45 240 450 145]) ;

h.ax2 = axes ('parent', h.Fig, ...
	'Units','pixels', ...
   'Position',[45 45 450 145]) ;

h.DrawVelocityText = uicontrol('style','text',...
   'position',[60 195 250 18],...
    'horizontalalignment','left',...
   'string','Draw a graph of the velocity of the train.',...
   'backgroundcolor', [.6 .8 1]) ;

h.DrawPositionText = uicontrol('style','text',...
   'position',[60 195 400 18],...
   'string', ...
   'Use the velocity graph to draw a graph of the position of the train.',...
   'horizontalalignment', 'left', ...
   'backgroundcolor', [.6 .8 1]) ;

h.answerbutton = uicontrol ('position', [505 50 110 30], ...
   'string', 'Answer',...
   'callback', 'position(''display_answer'')') ;

h.undo = uicontrol ('position', [505 85 110 30], ...
   'string', 'Undo',...
   'callback', 'position(''undo'')') ;

uicontrol ('position', [505 355 110 30],...
   'string', 'Clear',...
   'callback', 'position(''reset'')') 

uicontrol ('position', [505 320 110 30], ...
   'string', 'Help',...
   'callback', 'position(''help'')') 

uicontrol ('position', [505 285 110 30], ...
   'string', 'Quit',...
   'callback', 'position(''quit'')') 

