function result=tomsine(cmd);
% FREQUENCY AND AMPLITUDE
%
% A module to investigate A*sin(w*x) and its derivative as you vary
% the amplitude A and the frequency w.
%
% Enter A and w into the boxes, or click and drag the blue curve to
% change them dynamically.  The blue curve is the graph of A*sin(w*x) 
% and the red curve is the graph of its derivative.
%
% 2.1/22-July-1998

%tomsine
%
% Original by Julie Mitchell
% revised by Tom Insel (spring/summer 1998)
% new interface inspired by sigdemo2.m from the Mathworks.

if nargin<1,
    cmd='start';
end;

persistent freq amp fixed_x fixed_y time_line deriv_line freq_text amp_text

math_bkgnd = [0.6 0.8 1.0];
math_field = [1 1 1];
math_axis = [0.6 0.6 0.6];

init_freq = 2.5;
init_amp = 0.5;
min_freq = 0.05;
max_freq = 15;
min_amp = -2;
max_amp = 2;

if strcmp(cmd, 'start')

	control_figures = findobj('tag', 'tomsine_main');
	if length(control_figures) == 0
		control_figures = tomsine('makemain');
	end

	figure(control_figures(1));

elseif strcmp(cmd,'makemain')

    figNumber = figure(...
        'NumberTitle', 'off', ...
        'Name', 'Frequency and Amplitude', ...
		'backingstore','off',...
		'position', [20 20 600 400],...
		'resize', 'off',...
		'color', math_bkgnd,...
		'tag', 'tomsine_main');
 
	uicontrol(figNumber, ...
        'Style','frame', ...
        'Position',[10 85 580 1],...
        'BackgroundColor',math_bkgnd);
		
	uicontrol(figNumber,...
		'Style', 'text',...
		'Position', [10 55 200 20],...
		'string','PARAMETERS',...
		'horizontalalignment', 'left',...
		'backgroundcolor', math_bkgnd);

	uicontrol(figNumber,...
		'style', 'text',...
		'position', [10 35 80 20],...
		'horizontalalignment', 'right',...
		'backgroundcolor', math_bkgnd,...
		'string', 'frequency w =');

	freq_text = uicontrol(figNumber,...
		'style', 'edit',...
		'backgroundcolor', math_field,...
		'position', [100 35 100 20],...
		'callback', 'tomsine(''readparams'')',...
		'string', num2str(init_freq));

	uicontrol(figNumber,...
		'style', 'text',...
		'position', [10 10 80 20],...
		'horizontalalignment', 'right',...
		'backgroundcolor', math_bkgnd,...
		'string', 'amplitude A =');
		
	amp_text = uicontrol(figNumber,...
		'style', 'edit',...
		'backgroundcolor', math_field,...
		'position', [100 10 100 20],...
		'callback', 'tomsine(''readparams'')',...
		'string', num2str(init_amp));

    uicontrol(figNumber, ...
        'Style','push', ...
        'Position',[500 10 90 25], ...
        'String','Help', ...
        'Callback','helpwin tomsine');
 
	uicontrol(figNumber,...
		'Style','Pushbutton', ...
        'Position',[500 45 90 25], ...
        'Callback','tomsine(''done'')',...
		'String','Quit');
				
	uicontrol(figNumber,...
		'style', 'text',...
		'string', 'f(x) = A*sin(w*t)',...
		'foregroundcolor', 'b',...
		'backgroundcolor', math_bkgnd,...
		'position', [55 90 200 15]);

	uicontrol(figNumber,...
		'style', 'text',...
		'string', 'f''(x) = -w*A*cos(w*t)',...
		'foregroundcolor', 'r',...
		'backgroundcolor', math_bkgnd,...
		'position', [345 90 200 15]);

    axes('Position',[.05 .3 .9 .64])
	hold on
    time_line = plot(0, 0, 'ButtonDownFcn', 'tomsine(''down'')');
	deriv_line = plot(0,0,'r');
	plot([-pi pi],[0 0],'color',math_axis);
	hold off
    axis([-pi pi -3 3])
    xlabel('t')
    title('Click and drag the blue wave to change frequency and amplitude')
	
%	tomsine('redraw')
	tomsine('readparams')
	
	result = figNumber;
     
elseif strcmp(cmd,'down')

	set(time_line, 'erasemode', 'xor');
	set(deriv_line, 'erasemode', 'xor');

    pt=get(gca,'currentpoint');

    % find closest point on line to mouse click loc
    line_x = get(time_line, 'XData');
    line_y = get(time_line, 'YData');
    [temp,i] = min((line_x-pt(1,1)).^2 + (line_y-pt(1,2)).^2);	
    fixed_x = line_x(i);
    fixed_y = line_y(i);

    set(gcf,...
		'WindowButtonMotionFcn', 'tomsine(''move'')',...
		'WindowButtonUpFcn', 'tomsine(''up'')',...
		'pointer', 'crosshair');
	
elseif strcmp(cmd, 'readparams')

	freq_string = cleanup(get(freq_text, 'string'));
	amp_string = cleanup(get(amp_text, 'string'));

	err_field = 0;
	err_type = 0;
	
	eval([freq_string ';'], 'err_field=''frequency value''; err_type=''num'';');
	eval([amp_string ';'], 'err_field=''amplitude value''; err_type=''num'';');

	if err_field
		errmesg(err_field, err_type);
		error(lasterr);
	end

	freq = min(max_freq, max(min_freq, str2num(freq_string)));
	amp = min(max_amp, max(min_amp, str2num(amp_string)));
	
	set(freq_text, 'string', num2str(freq));
	set(amp_text, 'string', num2str(amp));
	
	tomsine('redraw')

elseif strcmp(cmd,'move') | strcmp(cmd, 'up')

    pt=get(gca,'currentpoint');
    x=pt(1,1);
    y=pt(1,2);

    amp1=y/fixed_y*amp;
    if (amp1>max_amp )
       amp1=max_amp;
    elseif (amp1<min_amp )
       amp1=min_amp;
    end;

    freq1=fixed_x/x*freq;
    if (freq1>max_freq)
       freq1=max_freq;
    elseif (freq1<min_freq)
       freq1=min_freq;
    end;
	
	if strcmp(cmd, 'move')
		t = linspace(-pi,pi,100);
	    set(time_line, 'YData', amp1*sin(freq1*t), 'XData', t);
		set(deriv_line, 'YData', -amp1*freq1*cos(freq1*t), 'XData', t);
		set(freq_text, 'string', num2str(freq1));
		set(amp_text, 'string', num2str(amp1));
	elseif strcmp(cmd, 'up')
	    set(gcf,...
			'WindowButtonMotionFcn', '',...
			'WindowButtonUpFcn', '',...
			'pointer', 'arrow');
		freq=freq1;
		amp=amp1;
		tomsine('redraw');
	end

elseif strcmp(cmd,'redraw')

	t = linspace(-pi,pi,500);
    set(time_line,...
		'YData', amp*sin(freq*t),...
		'XData', t,...
		'erasemode', 'normal')
	set(deriv_line,...
		'YData', -amp*freq*cos(freq*t),...
		'XData', t,...
		'erasemode', 'normal');
  
elseif strcmp(cmd,'done')

    close(gcf)

end
